@@ -24,7 +24,9 @@
 #include "kserviceaction.h"
 #include <QtCore/QStringList>
 #include <QtCore/QVariant>
+#ifndef KDE_NO_DEPRECATED
 #include <klibloader.h>
+#endif
 #include <kpluginfactory.h>
 #include <kpluginloader.h>
 #include <ksycocaentry.h>
@@ -53,7 +55,7 @@ class KServicePrivate;
  * @see KServiceGroup
  * @author Torben Weis
  */
-class KDECORE_EXPORT KService : public KSycocaEntry
+class KDECORE_EXPORT KService : public KSycocaEntry // TODO KDE5: inherit kshared, but move KSycocaEntry to Private
 {
 public:
     typedef KSharedPtr<KService> Ptr;
@@ -89,6 +91,8 @@ public:
 
     virtual ~KService();
 
+    // TODO KDE5: add mimeTypes() accessor
+
     /**
      * Services are either applications (executables) or dlopened libraries (plugins).
      * @return true if this service is an application, i.e. it has Type=Application in its
@@ -101,7 +105,9 @@ public:
      * @return the type of the service ("Application" or "Service")
      * @deprecated use isApplication()
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED QString type() const;
+#endif
 
     /**
      * Returns the executable.
@@ -166,7 +172,9 @@ public:
      * @return the path of the service's desktop file,
      *         or QString() if not set
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED QString desktopEntryPath() const;
+#endif
 
     /**
      * Returns the filename of the service desktop entry without any
@@ -272,7 +280,7 @@ public:
     bool hasServiceType( const QString& serviceTypePtr ) const;
 
     /**
-     * Checks whether the mime supports this mime type
+     * Checks whether the service supports this mime type
      * @param mimeTypePtr The name of the mime type you are
      *        interested in determining whether this service supports.
      *
@@ -287,8 +295,19 @@ public:
      * or one of the KService::serviceBy methods.
      *
      * @return true if the mime type you specified is supported, otherwise false.
+     * @deprecated, use hasMimeType(QString)
      */
-    bool hasMimeType( const KServiceType* mimeTypePtr ) const;
+#ifndef KDE_NO_DEPRECATED
+    KDE_DEPRECATED bool hasMimeType( const KServiceType* mimeTypePtr ) const;
+#endif
+
+    /**
+     * Checks whether the service supports this mime type
+     * @param mimeTypePtr The name of the mime type you are
+     *        interested in determining whether this service supports.
+     * @since 4.6
+     */
+    bool hasMimeType( const QString& mimeType ) const;
 
     /**
      * Set to true if it is allowed to use this service as the default (main)
@@ -336,8 +355,8 @@ public:
      * Whether the service should be shown in KDE at all
      * (including in context menus).
      * @return true if the service should be shown.
-     * 
-     * KMimeTypeTrader honours this and removes such services
+     *
+     * KMimeTypeTrader honors this and removes such services
      * from its results.
      *
      * @since 4.5
@@ -410,7 +429,9 @@ public:
      * @em Very @em important: Don't store the result in a KService* !
      * @deprecated there is never a good reason to use this method.
      */
+#ifndef KDE_NO_DEPRECATED
     static Ptr serviceByName( const QString& _name ); // KDE5: remove
+#endif
 
     /**
      * Find a service based on its path as returned by entryPath().
@@ -545,17 +566,20 @@ public:
     /**
      * @deprecated Use the non-static service->createInstance<T>(parent, args, &error)
      */
+#ifndef KDE_NO_DEPRECATED
     template <class T>
     static KDE_DEPRECATED T *createInstance(const KService::Ptr &service, QObject *parent = 0,
             const QVariantList &args = QVariantList(), QString *error = 0)
     {
         return service->createInstance<T>(parent, args, error);
     }
+#endif
 
     /**
      * @deprecated Use the non-static service->createInstance<T>(parent, args, &error)
      * where args is a QVariantList rather than a QStringList
      */
+#ifndef KDE_NO_DEPRECATED
     template <class T>
     static KDE_DEPRECATED T *createInstance( const KService::Ptr &service,
                               QObject *parent,
@@ -571,6 +595,7 @@ public:
 
         return KLibLoader::createInstance<T>( library, parent, args, error );
     }
+#endif
 
     /**
      * This template allows to create a component from a list of services,
@@ -587,6 +612,7 @@ public:
      * @return A pointer to the newly created object or a null pointer if the
      *         factory was unable to create an object of the given type.
      */
+#ifndef KDE_NO_DEPRECATED
     template <class T, class ServiceIterator>
     static KDE_DEPRECATED T *createInstance(ServiceIterator begin, ServiceIterator end, QObject *parent = 0,
             const QVariantList &args = QVariantList(), QString *error = 0)
@@ -607,7 +633,9 @@ public:
         }
         return 0;
     }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
     template <class T, class ServiceIterator>
     static KDE_DEPRECATED T *createInstance( ServiceIterator begin, ServiceIterator end,
                               QObject *parent,
@@ -627,6 +655,7 @@ public:
             *error = KLibLoader::ErrNoServiceFound;
         return 0;
     }
+#endif
 
 protected:
     friend class KMimeAssociations;
