@@ -18,9 +18,10 @@
  *  Boston, MA 02110-1301, USA.
  */
 
+#include "kmimetypetest.h"
+
 #include <config.h>
 #include <kdefakes.h>
-#include "kmimetypetest.h"
 #include <kde_file.h>
 #include <kprocess.h>
 #include <kmimetype.h>
@@ -36,9 +37,8 @@
 #include <kprotocolinfo.h>
 #include <kmimetypetrader.h>
 #include <kservicetypetrader.h>
-#include <kmimetypefactory.h>
+#include <kmimetyperepository_p.h>
 #include <ktemporaryfile.h>
-#include <ktempdir.h>
 #include <kdesktopfile.h>
 
 void KMimeTypeTest::initTestCase()
@@ -50,9 +50,25 @@ void KMimeTypeTest::initTestCase()
         // No need to run update-mime-database here, the dir is entirely gone.
     }
 
-    // Create fake text/plain part with a higher initial preference than katepart.
-    const QString fakePart = KStandardDirs::locateLocal("services", "faketextpart.desktop");
     bool mustUpdateKSycoca = false;
+
+    // Create fake text/x-patch part.
+    const QString fakePatchPart = KStandardDirs::locateLocal("services", "fakepatchpart.desktop");
+    const bool mustCreatePatchPart = !QFile::exists(fakePatchPart);
+    if (mustCreatePatchPart) {
+        mustUpdateKSycoca = true;
+        KDesktopFile file(fakePatchPart);
+        KConfigGroup group = file.desktopGroup();
+        group.writeEntry("Name", "FakePatchPart");
+        group.writeEntry("Type", "Service");
+        group.writeEntry("X-KDE-Library", "fakepatchpart");
+        group.writeEntry("ServiceTypes", "KParts/ReadOnlyPart");
+        group.writeEntry("MimeType", "text/x-diff;"); // Use an alias on purpose, to test if that works
+        group.writeEntry("InitialPreference", 5);
+    }
+
+    // Create fake text/plain part with a higher initial preference than the patch part.
+    const QString fakePart = KStandardDirs::locateLocal("services", "faketextpart.desktop");
     const bool mustCreate = !QFile::exists(fakePart);
     if (mustCreate) {
         mustUpdateKSycoca = true;
@@ -80,16 +96,40 @@ void KMimeTypeTest::initTestCase()
         group.writeEntry("MimeType", "text/plain;");
     }
 
+    // Create fake "NotShowIn=KDE" service
+    m_nonKdeApp = KStandardDirs::locateLocal("xdgdata-apps", "fake_nonkde_application.desktop");
+    const bool mustCreateNonKdeApp = !QFile::exists(m_nonKdeApp);
+    if (mustCreateNonKdeApp) {
+        mustUpdateKSycoca = true;
+        KDesktopFile file(m_nonKdeApp);
+        KConfigGroup group = file.desktopGroup();
+        group.writeEntry("Name", "NonKDEApp");
+        group.writeEntry("Type", "Application");
+        group.writeEntry("Exec", "xterm");
+        group.writeEntry("NotShowIn", "KDE;FVWM;");
+        group.writeEntry("MimeType", "text/plain;");
+        group.writeEntry("InitialPreference", "50");
+        group.writeEntry("Categories", "Qt;KDE;");
+    }
+
     if ( mustUpdateKSycoca ) {
         // Update ksycoca in ~/.kde-unit-test after creating the above
         QProcess::execute( KGlobal::dirs()->findExe(KBUILDSYCOCA_EXENAME) );
     }
+
+    KService::Ptr fakeApp = KService::serviceByStorageId("fake_nonkde_application.desktop");
+    QVERIFY(fakeApp); // it should be found.
+    QVERIFY(KService::serviceByDesktopPath(m_nonKdeApp)); // the desktoppath is the full path nowadays
+
+    KGlobal::locale();
 }
 
 void KMimeTypeTest::cleanupTestCase()
 {
     // If I want the konqueror unit tests to work, then I better not have a non-working part
     // as the preferred part for text/plain...
+    const QString fakePatchPart = KStandardDirs::locateLocal("services", "fakepatchpart.desktop");
+    QFile::remove(fakePatchPart);
     const QString fakePart = KStandardDirs::locateLocal("services", "faketextpart.desktop");
     QFile::remove(fakePart);
     const QString fakePlugin = KStandardDirs::locateLocal("services", "faketextplugin.desktop");
@@ -111,13 +151,10 @@ QTEST_KDEMAIN_CORE( KMimeTypeTest )
 
 void KMimeTypeTest::testByName()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     KMimeType::Ptr s0 = KMimeType::mimeType("application/x-zerosize");
     QVERIFY( s0 );
     QCOMPARE( s0->name(), QString::fromLatin1("application/x-zerosize") );
-    //qDebug("Comment is %s", qPrintable(s0->comment()) );
+    QCOMPARE( s0->comment(), QString::fromLatin1("empty document") );
 
     KMimeType::Ptr s0Again = KMimeType::mimeType("application/x-zerosize");
     QCOMPARE(s0Again->name(), s0->name());
@@ -131,8 +168,14 @@ void KMimeTypeTest::testByName()
     KMimeType::Ptr krita = KMimeType::mimeType("application/x-krita");
     QVERIFY( krita );
 
-    KMimeType::Ptr bzip2 = KMimeType::mimeType("application/x-bzip2", KMimeType::ResolveAliases);
+    // Test <comment> parsing with application/rdf+xml which has the english comment after the other ones
+    KMimeType::Ptr rdf = KMimeType::mimeType("application/rdf+xml");
+    QVERIFY(rdf);
+    QCOMPARE(rdf->comment(), QString::fromLatin1("RDF file"));
+
+    KMimeType::Ptr bzip2 = KMimeType::mimeType("application/x-bzip2");
     QVERIFY( bzip2 );
+    QCOMPARE(bzip2->comment(), QString::fromLatin1("Bzip archive"));
 
     KMimeType::Ptr defaultMime = KMimeType::mimeType("application/octet-stream");
     QVERIFY(defaultMime);
@@ -141,9 +184,6 @@ void KMimeTypeTest::testByName()
 
 void KMimeTypeTest::testIcons()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     checkIcon( KUrl( "file:/tmp/" ), "inode-directory" );
 
     if ( !KUser().isSuperUser() ) // Can't test this one if running as root
@@ -160,14 +200,13 @@ void KMimeTypeTest::testIcons()
 
 void KMimeTypeTest::testFindByPathUsingFileName_data()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
     QTest::addColumn<QString>("fileName");
     QTest::addColumn<QString>("expectedMimeType");
     // Maybe we could also add a expectedAccuracy column...
 
     QTest::newRow("text") << "textfile.txt" << "text/plain";
     QTest::newRow("case-insensitive search") << "textfile.TxT" << "text/plain";
+    QTest::newRow("case-insensitive match on a non-lowercase glob") << "foo.z" << "application/x-compress";
 
     QTest::newRow("case-sensitive uppercase match") << "textfile.C" << "text/x-c++src";
     QTest::newRow("case-sensitive lowercase match") << "textfile.c" << "text/x-csrc";
@@ -193,12 +232,12 @@ void KMimeTypeTest::testFindByPathUsingFileName_data()
     QTest::newRow("doesn't exist but has known extension") << "IDontExist.txt" << "text/plain";
 
     // Can't use KIconLoader since this is a "without GUI" test.
-    if (!KStandardDirs::locate("icon", "oxygen/").isEmpty()) {
-        QString fh = KStandardDirs::locate("icon", "oxygen/22x22/places/folder.png");
-        QVERIFY(!fh.isEmpty()); // if the file doesn't exist, please fix the above to point to an existing icon
-        QTest::newRow("png image") << fh << "image/png";
+    if (KStandardDirs::locate("icon", "oxygen/").isEmpty()) {
+        kWarning() << "oxygen not found";
     } else {
-        kWarning() << "oxygen icon theme not found";
+        QString fh = KStandardDirs::locate( "icon", "oxygen/22x22/places/folder.png" );
+        QVERIFY( !fh.isEmpty() ); // if the file doesn't exist, please fix the above to point to an existing icon
+        QTest::newRow("png image") << fh << "image/png";
     }
 
     QString exePath = KStandardDirs::findExe( "kioexec" );
@@ -232,6 +271,9 @@ void KMimeTypeTest::testAdditionalGlobs_data()
     QTest::addColumn<bool>("expected");
 
     QTest::newRow("one star, match") << "foo.txt" << "*.txt" << true;
+    QTest::newRow("README*, match") << "README.foo" << "README*" << true;
+    QTest::newRow("README.*, match") << "README.foo" << "README.*" << true;
+    QTest::newRow("README.*, no match") << "README" << "README.*" << false;
     QTest::newRow("two stars, match") << "andre.ts.001" << "*.ts.0*" << true;
     QTest::newRow("two stars, no match") << "andre.ts" << "*.ts.0*" << false;
 }
@@ -242,17 +284,13 @@ void KMimeTypeTest::testAdditionalGlobs()
     QFETCH(QString, pattern);
     QFETCH(bool, expected);
 
-    QCOMPARE(KMimeTypeFactory::matchFileName(filename, pattern), expected);
+    QCOMPARE(KMimeTypeRepository::matchFileName(filename, pattern), expected);
 }
 
 // All the simple tests for findByPath are in testFindByPathUsingFileName_data.
 // In here we do the tests that need some content in a temporary file.
 void KMimeTypeTest::testFindByPathWithContent()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
-
     KMimeType::Ptr mime;
 
     // Test a real PDF file.
@@ -328,7 +366,6 @@ void KMimeTypeTest::testFindByUrl()
     mime = KMimeType::findByUrl(KUrl("fish://host/test1")); // like fish does, to test for known extensions
     QVERIFY(mime);
     QCOMPARE(mime->name(), QString::fromLatin1("application/octet-stream"));
-
 }
 
 void KMimeTypeTest::testFindByNameAndContent()
@@ -442,10 +479,7 @@ void KMimeTypeTest::testFindByFileContent()
 
 void KMimeTypeTest::testAllMimeTypes()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
-    const KMimeType::List lst = KMimeType::allMimeTypes();
+    const KMimeType::List lst = KMimeType::allMimeTypes(); // does NOT include aliases
     QVERIFY( !lst.isEmpty() );
 
     for ( KMimeType::List::ConstIterator it = lst.begin();
@@ -459,15 +493,12 @@ void KMimeTypeTest::testAllMimeTypes()
 
         const KMimeType::Ptr lookedupMime = KMimeType::mimeType( name );
         QVERIFY( lookedupMime ); // not null
-        QCOMPARE( lookedupMime->name(), name );
+        QCOMPARE( lookedupMime->name(), name ); // if this fails, you have an alias defined as a real mimetype too!
     }
 }
 
 void KMimeTypeTest::testAlias()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     const KMimeType::Ptr canonical = KMimeType::mimeType( "application/xml" );
     QVERIFY( canonical );
     KMimeType::Ptr alias = KMimeType::mimeType("text/xml", KMimeType::DontResolveAlias);
@@ -478,13 +509,14 @@ void KMimeTypeTest::testAlias()
 
     QVERIFY(alias->is("application/xml"));
     QVERIFY(canonical->is("text/xml"));
+
+    // Test for bug 197346: does nspluginscan see that audio/mp3 already exists?
+    bool mustWriteMimeType = KMimeType::mimeType("audio/mp3").isNull();
+    QVERIFY(!mustWriteMimeType);
 }
 
 void KMimeTypeTest::testMimeTypeParent()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     // All file-like mimetypes inherit from octet-stream
     const KMimeType::Ptr wordperfect = KMimeType::mimeType("application/vnd.wordperfect");
     QVERIFY(wordperfect);
@@ -506,7 +538,7 @@ void KMimeTypeTest::testMimeTypeParent()
     QCOMPARE(directory->parentMimeTypes().count(), 0);
     QVERIFY(!directory->is("application/octet-stream"));
 
-    // Check that text/x-patch knows that it inherits from text/plain (it says so explicitely)
+    // Check that text/x-patch knows that it inherits from text/plain (it says so explicitly)
     const KMimeType::Ptr plain = KMimeType::mimeType( "text/plain" );
     const KMimeType::Ptr derived = KMimeType::mimeType( "text/x-patch" );
     QVERIFY( derived );
@@ -571,6 +603,8 @@ void KMimeTypeTest::testMimeTypeInheritancePerformance()
     // 0.57 msec / 1,115,000 ticks / 938,356 instr. loads per iteration
     // After converting the QMap for aliases into a QHash too:
     // 0.48 msec / 960,000 ticks / 791,404 instr. loads per iteration
+    // July 2010: After moving KMimeType out of ksycoca:
+    // 0.21 msec / 494,000 ticks / 568,345 instr. loads per iteration
 }
 
 // Helper method for all the trader tests
@@ -599,8 +633,8 @@ void KMimeTypeTest::testMimeTypeTraderForTextPlain()
 
     // Querying mimetype trader for services associated with text/plain
     KService::List offers = KMimeTypeTrader::self()->query("text/plain", "KParts/ReadOnlyPart");
-    QVERIFY( offerListHasService( offers, "katepart.desktop" ) );
-    QVERIFY( offerListHasService( offers, "faketextpart.desktop" ) );
+    QVERIFY(!offerListHasService(offers, "fakepatchpart.desktop"));
+    QVERIFY(offerListHasService(offers, "faketextpart.desktop"));
 
     offers = KMimeTypeTrader::self()->query("text/plain", "KTextEditor/Plugin");
     QVERIFY( offers.count() > 0 );
@@ -610,8 +644,8 @@ void KMimeTypeTest::testMimeTypeTraderForTextPlain()
     QVERIFY( offerListHasService( offers, "faketextplugin.desktop" ) );
 
     // We shouldn't have non-plugins
-    QVERIFY( !offerListHasService( offers, "katepart.desktop" ) );
-
+    QVERIFY( !offerListHasService( offers, "fakepatchpart.desktop" ) );
+    QVERIFY( !offerListHasService( offers, "faketextpart.desktop" ) );
 }
 
 void KMimeTypeTest::testMimeTypeTraderForDerivedMimeType()
@@ -621,7 +655,7 @@ void KMimeTypeTest::testMimeTypeTraderForDerivedMimeType()
 
     // Querying mimetype trader for services associated with text/x-patch, which inherits from text/plain
     KService::List offers = KMimeTypeTrader::self()->query("text/x-patch", "KParts/ReadOnlyPart");
-    QVERIFY( offerListHasService( offers, "katepart.desktop" ) );
+    QVERIFY( offerListHasService( offers, "fakepatchpart.desktop" ) );
     QVERIFY( offerListHasService( offers, "faketextpart.desktop" ) );
     QVERIFY( (*offers.begin())->entryPath() != "faketextpart.desktop" ); // in the list, but not preferred
 
@@ -633,9 +667,21 @@ void KMimeTypeTest::testMimeTypeTraderForDerivedMimeType()
     QVERIFY( offerListHasService( offers, "faketextplugin.desktop" ) );
 
     offers = KMimeTypeTrader::self()->query("text/x-patch", "Application");
-    QVERIFY( !offerListHasService( offers, "katepart.desktop" ) );
+    QVERIFY( !offerListHasService( offers, "faketextpart.desktop" ) );
+
+    // We shouldn't have non-kde apps
+    Q_FOREACH( KService::Ptr service, offers )
+        kDebug() << service->name() << service->entryPath();
+
+    QVERIFY( !offerListHasService( offers, m_nonKdeApp ) );
 }
 
+void KMimeTypeTest::testPreferredService()
+{
+    // The "NotShowIn=KDE" service should not be the preferred one!
+    KService::Ptr serv = KMimeTypeTrader::self()->preferredService("text/plain");
+    QVERIFY( serv->entryPath() != m_nonKdeApp );
+}
 
 void KMimeTypeTest::testMimeTypeTraderForAlias()
 {
@@ -643,49 +689,51 @@ void KMimeTypeTest::testMimeTypeTraderForAlias()
         QSKIP( "ksycoca not available", SkipAll );
 
     const KService::List referenceOffers = KMimeTypeTrader::self()->query("application/xml", "KParts/ReadOnlyPart");
-    QVERIFY( offerListHasService( referenceOffers, "katepart.desktop" ) );
+    QVERIFY(offerListHasService(referenceOffers, "faketextpart.desktop"));
+    QVERIFY(!offerListHasService(referenceOffers, "fakepatchpart.desktop"));
 
     // Querying mimetype trader for services associated with text/xml, which is an alias for application/xml
     const KService::List offers = KMimeTypeTrader::self()->query("text/xml", "KParts/ReadOnlyPart");
-    QVERIFY( offerListHasService( offers, "katepart.desktop" ) );
+    QVERIFY(offerListHasService(offers, "faketextpart.desktop"));
+    QVERIFY(!offerListHasService(offers, "fakepatchpart.desktop"));
 
     QCOMPARE(offers.count(), referenceOffers.count());
 }
 
 void KMimeTypeTest::testHasServiceType1() // with services constructed with a full path (rare)
 {
-    QString katepartPath = KStandardDirs::locate( "services", "katepart.desktop" );
-    QVERIFY( !katepartPath.isEmpty() );
-    KService katepart( katepartPath );
-    QVERIFY( katepart.hasMimeType( KMimeType::mimeType( "text/plain" ).data() ) );
-    //QVERIFY( katepart.hasMimeType( KMimeType::mimeType( "text/x-patch" ).data() ) ); // inherited mimetype; fails
-    QVERIFY( !katepart.hasMimeType( KMimeType::mimeType( "image/png" ).data() ) );
-    QVERIFY( katepart.hasServiceType( "KParts/ReadOnlyPart" ) );
-    QVERIFY( katepart.hasServiceType( "KParts/ReadWritePart" ) );
-    QVERIFY( !katepart.hasServiceType( "KTextEditor/Plugin" ) );
+    QString faketextpartPath = KStandardDirs::locate( "services", "faketextpart.desktop" );
+    QVERIFY( !faketextpartPath.isEmpty() );
+    KService faketextpart( faketextpartPath );
+    QVERIFY( faketextpart.hasMimeType( "text/plain" ) );
+    QVERIFY(!faketextpart.hasMimeType("text/x-patch")); // inherited mimetype; fails
+    QVERIFY( !faketextpart.hasMimeType( "image/png" ) );
+    QVERIFY( faketextpart.hasServiceType( "KParts/ReadOnlyPart" ) );
+    QVERIFY( !faketextpart.hasServiceType( "KParts/ReadWritePart" ) );
+    QVERIFY( !faketextpart.hasServiceType( "KTextEditor/Plugin" ) );
 
-    QString ktexteditor_insertfilePath = KStandardDirs::locate( "services", "ktexteditor_insertfile.desktop" );
-    QVERIFY( !ktexteditor_insertfilePath.isEmpty() );
-    KService ktexteditor_insertfile( ktexteditor_insertfilePath );
-    QVERIFY( ktexteditor_insertfile.hasServiceType( "KTextEditor/Plugin" ) );
-    QVERIFY( !ktexteditor_insertfile.hasServiceType( "KParts/ReadOnlyPart" ) );
+    QString textPluginPath = KStandardDirs::locate( "services", "faketextplugin.desktop" );
+    QVERIFY( !textPluginPath.isEmpty() );
+    KService textPlugin( textPluginPath );
+    QVERIFY( textPlugin.hasServiceType( "KTextEditor/Plugin" ) );
+    QVERIFY( !textPlugin.hasServiceType( "KParts/ReadOnlyPart" ) );
 }
 
 void KMimeTypeTest::testHasServiceType2() // with services coming from ksycoca
 {
-    KService::Ptr katepart = KService::serviceByDesktopPath( "katepart.desktop" );
-    QVERIFY( !katepart.isNull() );
-    QVERIFY( katepart->hasMimeType( KMimeType::mimeType( "text/plain" ).data() ) );
-    QVERIFY( katepart->hasMimeType( KMimeType::mimeType( "text/x-patch" ).data() ) ); // due to inheritance
-    QVERIFY( !katepart->hasMimeType( KMimeType::mimeType( "image/png" ).data() ) );
-    QVERIFY( katepart->hasServiceType( "KParts/ReadOnlyPart" ) );
-    QVERIFY( katepart->hasServiceType( "KParts/ReadWritePart" ) );
-    QVERIFY( !katepart->hasServiceType( "KTextEditor/Plugin" ) );
+    KService::Ptr faketextpart = KService::serviceByDesktopPath( "faketextpart.desktop" );
+    QVERIFY( !faketextpart.isNull() );
+    QVERIFY( faketextpart->hasMimeType( "text/plain" ) );
+    QVERIFY( faketextpart->hasMimeType( "text/x-patch" ) ); // due to inheritance
+    QVERIFY( !faketextpart->hasMimeType( "image/png" ) );
+    QVERIFY( faketextpart->hasServiceType( "KParts/ReadOnlyPart" ) );
+    QVERIFY( !faketextpart->hasServiceType( "KParts/ReadWritePart" ) );
+    QVERIFY( !faketextpart->hasServiceType( "KTextEditor/Plugin" ) );
 
-    KService::Ptr ktexteditor_insertfile = KService::serviceByDesktopPath( "ktexteditor_insertfile.desktop" );
-    QVERIFY( !ktexteditor_insertfile.isNull() );
-    QVERIFY( ktexteditor_insertfile->hasServiceType( "KTextEditor/Plugin" ) );
-    QVERIFY( !ktexteditor_insertfile->hasServiceType( "KParts/ReadOnlyPart" ) );
+    KService::Ptr textPlugin= KService::serviceByDesktopPath( "faketextplugin.desktop" );
+    QVERIFY( !textPlugin.isNull() );
+    QVERIFY( textPlugin->hasServiceType( "KTextEditor/Plugin" ) );
+    QVERIFY( !textPlugin->hasServiceType( "KParts/ReadOnlyPart" ) );
 }
 
 void KMimeTypeTest::testPatterns_data()
@@ -702,7 +750,7 @@ void KMimeTypeTest::testPatterns_data()
     QTest::newRow("oasis presentation mimetype") << "application/vnd.oasis.opendocument.presentation" << "*.odp" << ".odp";
     QTest::newRow("mimetype with multiple patterns, *.doc added by kde") << "text/plain" << "*.asc;*.txt;*.doc;*,v" << ".txt";
     QTest::newRow("mimetype with uncommon pattern") << "application/x-kcachegrind" << "callgrind.out*;cachegrind.out*" << QString();
-    QTest::newRow("mimetype with no patterns") << "application/pkcs7-mime" << QString() << QString();
+    QTest::newRow("mimetype with no patterns") << "application/x-ole-storage" << QString() << QString();
 }
 
 void KMimeTypeTest::testPatterns()
@@ -716,6 +764,15 @@ void KMimeTypeTest::testPatterns()
     QStringList expectedPatterns = patterns.split(';');
     expectedPatterns.sort();
     QStringList mimePatterns = mime->patterns();
+
+    if (mimeType == "application/vnd.oasis.opendocument.text" && mimePatterns.contains("*.fodt")) {
+        QSKIP("Skipping test which would fail due to an upstream bug, see https://bugs.freedesktop.org/show_bug.cgi?id=31242", SkipSingle);
+    }
+
+    if (mimeType == "application/vnd.oasis.opendocument.presentation" && mimePatterns.contains("*.fodp")) {
+        QSKIP("Skipping test which would fail due to an upstream bug, see https://bugs.freedesktop.org/show_bug.cgi?id=31242", SkipSingle);
+    }
+
     // shared-mime-info 0.30 adds *,v to text/plain, let's add it from this test so that it works
     // with older versions too.
     if (mimeType == "text/plain" && !mimePatterns.contains("*,v"))
@@ -807,7 +864,7 @@ void KMimeTypeTest::testParseMagicFile_data()
 
     QBuffer magicBuffer(&magicData);
     magicBuffer.open(QIODevice::ReadOnly);
-    m_rules = KMimeTypeFactory::self()->parseMagicFile(&magicBuffer, "magicData");
+    m_rules = KMimeTypeRepository::self()->parseMagicFile(&magicBuffer, "magicData");
     QCOMPARE(m_rules.count(), 7);
 
     const KMimeMagicRule rule = m_rules[0];
@@ -841,10 +898,11 @@ void KMimeTypeTest::testParseMagicFile()
     QBuffer testBuffer;
     testBuffer.setData(testData.toLatin1());
     QVERIFY(testBuffer.open(QIODevice::ReadOnly));
+    const qint64 testBufferSize = testBuffer.size();
     QString found;
     QByteArray beginning;
     Q_FOREACH(const KMimeMagicRule& rule, m_rules) {
-        if (rule.match(&testBuffer, beginning)) {
+        if (rule.match(&testBuffer, testBufferSize, beginning)) {
             found = rule.mimetype();
             break;
         }
@@ -853,4 +911,51 @@ void KMimeTypeTest::testParseMagicFile()
     testBuffer.close();
 }
 
+void KMimeTypeTest::testHelperProtocols()
+{
+    QVERIFY(KProtocolInfo::isKnownProtocol("mailto"));
+    QVERIFY(KProtocolInfo::isHelperProtocol("mailto"));
+    QVERIFY(KProtocolInfo::isHelperProtocol(KUrl("mailto:faure@kde.org")));
+    QVERIFY2(KProtocolInfo::exec("mailto").contains(QLatin1String("kmail -caption \"%c\"")), // comes from KMail2.desktop
+                qPrintable(KProtocolInfo::exec("mailto")));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("http"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("ftp"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("file"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("unknown"));
+    QVERIFY(KProtocolInfo::isHelperProtocol("telnet"));
+
+    // To test that compat still works
+    if (KProtocolInfo::isKnownProtocol("tel")) {
+        QVERIFY(KProtocolInfo::isHelperProtocol("tel"));
+    }
+}
+
+void KMimeTypeTest::testFromThread()
+{
+    // Some simple tests to test more API from testThreads without using _data()
+    KMimeType::Ptr mime = KMimeType::mimeType("application/pdf");
+    QVERIFY(mime);
+    QCOMPARE(mime->mainExtension(), QString::fromLatin1(".pdf"));
+}
+
+void KMimeTypeTest::testThreads()
+{
+    QThreadPool::globalInstance()->setMaxThreadCount(20);
+    // Note that data-based tests cannot be used here (QTest::fetchData asserts).
+    QList<QFuture<void> > futures;
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByUrl);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByFileContent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByNameAndContent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByPathWithContent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testAllMimeTypes);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testAlias);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testMimeTypeParent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testPreferredService);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFromThread);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testHelperProtocols);
+    kDebug() << "Joining all threads";
+    Q_FOREACH(QFuture<void> f, futures) // krazy:exclude=foreach
+        f.waitForFinished();
+}
+
 #include "kmimetypetest.moc"
