@@ -1,3 +1,4 @@
+// krazy:excludeall=qclasses
 /* This file is part of the KDE libraries
     Copyright (c) 1999-2005 Waldo Bastian <bastian@kde.org>
     Copyright (c) 2000-2005 David Faure <faure@kde.org>
@@ -18,8 +19,9 @@
 */
 // -*- mode: c++; c-basic-offset: 2 -*-
 
-#include "qtest_kde.h"
 #include "kurltest.h"
+
+#include "qtest_kde.h"
 #include "kurltest.moc"
 
 QTEST_KDEMAIN_CORE( KUrlTest )
@@ -38,6 +40,9 @@ QTEST_KDEMAIN_CORE( KUrlTest )
 #include <stdio.h>
 #include <stdlib.h>
 
+//QCOMPARE cannot be used to strictly check for empty or null QString as it treats QString("") == QString()
+#define QSTREMPTY(_str) QVERIFY(!_str.isNull() && _str.isEmpty())
+
 void KUrlTest::testEmptyURL()
 {
   KUrl emptyURL;
@@ -104,6 +109,23 @@ void KUrlTest::testSetQuery()
   QCOMPARE( url1.query(), QString() );
 }
 
+void KUrlTest::testEmptyNullReference()
+{
+  KUrl url1 = KUrl("http://www.kde.org");
+  QVERIFY( !url1.hasRef() );
+  QVERIFY( !url1.hasHTMLRef() );
+  QVERIFY( url1.ref().isNull() );
+  QVERIFY( url1.htmlRef().isNull() );
+  QVERIFY( url1.encodedHtmlRef().isNull() );
+
+  url1 = "http://www.kde.org#";
+  QVERIFY( url1.hasRef() );
+  QVERIFY( url1.hasHTMLRef() );
+  QSTREMPTY( url1.ref() );
+  QSTREMPTY( url1.htmlRef() );
+  QSTREMPTY( url1.encodedHtmlRef() );
+}
+
 void KUrlTest::testSetRef()
 {
   KUrl url1 = KUrl( QByteArray( "http://www.kde.org/foo.cgi#foo=bar" ) );
@@ -117,9 +139,12 @@ void KUrlTest::testSetRef()
   QCOMPARE( url1.ref(), QString("#" ) );
 #endif
   url1.setRef( "" );
-  QCOMPARE( url1.ref(), QString("" ) );
+  QSTREMPTY( url1.ref() );
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi#") );
+
   url1.setRef( QString() );
-  QCOMPARE( url1.ref(), QString() );
+  QVERIFY( url1.ref().isNull() );
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi") );
 }
 
 void KUrlTest::testSetHTMLRef()
@@ -134,9 +159,12 @@ void KUrlTest::testSetHTMLRef()
   QCOMPARE( url1.htmlRef(), QString("#") );
   QCOMPARE( url1.ref(), QString("%23") ); // it's encoded
   url1.setHTMLRef( "" );
-  QCOMPARE( url1.htmlRef(), QString("") );
+  QSTREMPTY( url1.htmlRef() );
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi#") );
+
   url1.setHTMLRef( QString() );
-  QCOMPARE( url1.htmlRef(), QString() );
+  QVERIFY( url1.htmlRef().isNull() );
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi") );
 }
 
 void KUrlTest::testQUrl()
@@ -147,11 +175,23 @@ void KUrlTest::testQUrl()
   QUrl url2( "file:///c:/home/dfaure/my#%2f" );
   QCOMPARE( url2.toString(), QString( "file:///c:/home/dfaure/my#%2f" ) );
 #endif
+
+  // Show how toString() is confusing
+  QUrl url3 = QUrl::fromLocalFile( "/home/dfaure/hash#file" );
+  QCOMPARE( url3.toString(), QString( "file:///home/dfaure/hash#file" ) ); // ouch
+  QString url3Str = url3.toString();
+  QUrl url4(url3Str);
+  QCOMPARE( url4.toString(), url3Str );
+  QVERIFY( url3 != url4 ); // unexpected, huh?
+  //QCOMPARE( QString::fromLatin1(url4.toEncoded()), QString::fromLatin1(url3.toEncoded()) ); // fails
 }
 
 
 void KUrlTest::testIsLocalFile()
 {
+  KUrl trash("trash:/");
+  QVERIFY(!trash.isLocalFile());
+
   KUrl local_file_1("file://localhost/my/file");
   QVERIFY( local_file_1.isLocalFile() );
 
@@ -401,7 +441,7 @@ void KUrlTest::testEmptyQueryOrRef()
   QVERIFY( waba1.hasRef() );
   QVERIFY( waba1.hasFragment() );
   QVERIFY( waba1.hasHTMLRef() );
-  QCOMPARE( waba1.encodedHtmlRef(), QString() );
+  QSTREMPTY( waba1.encodedHtmlRef() );
   //qurl = QUrl::fromEncoded("http://www.kde.org/cgi/test.cgi#", QUrl::TolerantMode);
   //QCOMPARE( qurl.toEncoded(), QByteArray("http://www.kde.org/cgi/test.cgi#") );
 
@@ -646,11 +686,11 @@ void KUrlTest::testSetFileName() // and addPath
   QString pre;
 #ifdef Q_OS_WIN
   // On Windows we explicitly prepend a slash, see KUrl::setPath
-  pre = "/";
+  pre = '/';
 #endif
   singleFile.setFileName( "bar.bin" );
-  QCOMPARE( singleFile.path(), pre + QString("bar.bin") );
-  QCOMPARE( singleFile.pathOrUrl(), pre + QString("bar.bin") );
+  QCOMPARE( singleFile.path(), QString(pre + "bar.bin") );
+  QCOMPARE( singleFile.pathOrUrl(), QString(pre + "bar.bin") );
 }
 
 void KUrlTest::testDirectory()
@@ -1202,17 +1242,17 @@ void KUrlTest::testSubURL()
   //QCOMPARE( url1.htmlRef(), QString("myref") );
   QCOMPARE( url1.upUrl().url(), QString("file:///home/dfaure/") );
 
-  const KUrl::List splitted = KUrl::split( url1 );
-  QCOMPARE( splitted.count(), 3 );
-  //kDebug() << splitted.toStringList();
-  QCOMPARE( splitted[0].url(), QString("file:///home/dfaure/my%20tar%20file.tgz#myref") );
-  QCOMPARE( splitted[1].url(), QString("gzip:/#myref") );
-  QCOMPARE( splitted[2].url(), QString("tar:/#myref") );
+  const KUrl::List splitList = KUrl::split( url1 );
+  QCOMPARE( splitList.count(), 3 );
+  //kDebug() << splitList.toStringList();
+  QCOMPARE( splitList[0].url(), QString("file:///home/dfaure/my%20tar%20file.tgz#myref") );
+  QCOMPARE( splitList[1].url(), QString("gzip:/#myref") );
+  QCOMPARE( splitList[2].url(), QString("tar:/#myref") );
 
 #if QT_VERSION < 0x040500
   QSKIP( "Multiple sub urls not supported with Qt < 4.5", SkipSingle );
 #endif
-  KUrl rejoined = KUrl::join(splitted);
+  KUrl rejoined = KUrl::join(splitList);
   QCOMPARE(rejoined.url(), url1.url());
 
   u1 = "error:/?error=14&errText=Unknown%20host%20asdfu.adgi.sdfgoi#http://asdfu.adgi.sdfgoi";
@@ -1229,7 +1269,7 @@ void KUrlTest::testSubURL()
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
-  QCOMPARE( url1.htmlRef(), QString("") );
+  QVERIFY( url1.htmlRef().isNull() );
   QCOMPARE( url1.upUrl().url(), QString("file:///home/dfaure/") );
 
   u1 = "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
@@ -1238,7 +1278,7 @@ void KUrlTest::testSubURL()
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
-  QCOMPARE( url1.htmlRef(), QString("") );
+  QVERIFY( url1.htmlRef().isNull() );
   QCOMPARE( url1.upUrl().url(), QString("file:///home/dfaure/") );
 
 #if 0
@@ -1249,7 +1289,7 @@ void KUrlTest::testSubURL()
   QVERIFY( !url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
-  QCOMPARE( url1.htmlRef(), QString("") );
+  QVERIFY( url1.htmlRef().isNull() );
   QCOMPARE( url1.upUrl().url(), QString("file:///home/dfaure/cdrdao-1.1.5/dao/#CdrDriver.cc#") );
 #endif
 
@@ -1259,7 +1299,7 @@ void KUrlTest::testSubURL()
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
-  QCOMPARE( url1.htmlRef(), QString("") );
+  QVERIFY( url1.htmlRef().isNull() );
   const KUrl::List url1Splitted = KUrl::split( url1 );
   QCOMPARE( url1Splitted.count(), 3 );
   //kDebug() << url1Splitted.toStringList();
@@ -1319,8 +1359,10 @@ void KUrlTest::testComparisons()
 
   QString ucmp1 = "ftp://ftp.de.kde.org/dir";
   QString ucmp2 = "ftp://ftp.de.kde.org/dir/";
+#ifndef KDE_NO_DEPRECATED
   QVERIFY(!urlcmp(ucmp1, ucmp2));
   QVERIFY(urlcmp(ucmp1, ucmp2, KUrl::CompareWithoutTrailingSlash)); //only slash difference, ignore_trailing
+#endif
 
   {
       KUrl u1(ucmp1);
@@ -1334,9 +1376,11 @@ void KUrlTest::testComparisons()
   {
       QString str1 = QString::fromLatin1( "ftp://ftp.de.kde.org" );
       QString str2 = QString::fromLatin1( "ftp://ftp.de.kde.org/" );
+#ifndef KDE_NO_DEPRECATED
       QVERIFY(!urlcmp(str1, str2));
       QVERIFY(!urlcmp(str1, str2, KUrl::CompareWithoutTrailingSlash)); // empty path != '/'
       QVERIFY(urlcmp(str1, str2, KUrl::CompareWithoutTrailingSlash|KUrl::AllowEmptyPath));
+#endif
       KUrl u1(str1);
       KUrl u2(str2);
       QVERIFY(!u1.equals(u2));
@@ -1345,6 +1389,7 @@ void KUrlTest::testComparisons()
   }
 
   QString ucmp3 = "ftp://ftp.de.kde.org/dir/#";
+#ifndef KDE_NO_DEPRECATED
   QVERIFY( !urlcmp(ucmp2,ucmp3) ); // (only hash difference)
   QVERIFY( urlcmp(ucmp2,ucmp3,KUrl::CompareWithoutFragment) ); // (only hash difference, ignore_ref)
   QVERIFY( urlcmp(ucmp2,ucmp3,KUrl::CompareWithoutTrailingSlash | KUrl::CompareWithoutFragment) ); // (slash and hash difference, ignore_trailing, ignore_ref)
@@ -1354,6 +1399,7 @@ void KUrlTest::testComparisons()
   QVERIFY( !urlcmp("",ucmp1,KUrl::CompareWithoutFragment) ); // (empty, not empty)
   QVERIFY( !urlcmp("file",ucmp1) ); // (malformed, not empty)
   QVERIFY( !urlcmp("file",ucmp1,KUrl::CompareWithoutFragment) ); // (malformed, not empty)
+#endif
 
   KUrl ftpUrl( "ftp://ftp.de.kde.org" );
   QCOMPARE( ftpUrl.path(), QString());
@@ -1636,6 +1682,14 @@ void KUrlTest::testMailto()
   QUrl qurl("mailto:null@kde.org?subject=hello#world"); // #80165: is #world part of fragment or query? RFC-3986 says: fragment.
   QCOMPARE(QString::fromLatin1(qurl.encodedQuery()), QString("subject=hello"));
 
+  {
+      KUrl mailtoUrl;
+      mailtoUrl.setProtocol("mailto");
+      mailtoUrl.setPath("a%b");
+      QCOMPARE(mailtoUrl.path(), QString("a%b"));
+      QCOMPARE(mailtoUrl.url(), QString("mailto:a%25b"));
+  }
+
 #if 0
   // I wrote this test in the very first kurltest, but there's no proof that it's actually valid.
   // Andreas says this is broken, i.e. against rfc2368.
@@ -1755,7 +1809,7 @@ void KUrlTest::testUtf8()
   // so it's ok to use that to encode urls as filenames (e.g. in kio_http cache)
   KUrl fractionSlash(QString("http://kde.org/a")+QChar(0x2044)+"b");
   QCOMPARE(fractionSlash.url(), QString("http://kde.org/a%E2%81%84b"));
-  QCOMPARE(fractionSlash.prettyUrl(), QString("http://kde.org/a")+QChar(0x2044)+"b");
+  QCOMPARE(fractionSlash.prettyUrl(), QString(QString("http://kde.org/a")+QChar(0x2044)+"b"));
 }
 
 void KUrlTest::testOtherEncodings()
@@ -1975,3 +2029,87 @@ void KUrlTest::testToLocalFile()
   QCOMPARE( urlWithHost.toLocalFile(), localFile );
   QCOMPARE( urlWithoutHost.toLocalFile(), localFile );
 }
+
+void KUrlTest::testUrl_data()
+{
+    QTest::addColumn<KUrl>( "url" );
+    QTest::addColumn<QString>( "urlLTS" );
+    QTest::addColumn<QString>( "urlRTS" );
+    QTest::addColumn<QString>( "urlATS" );
+
+    QTest::newRow("local file 1")
+        << KUrl("file:///")
+        << QString::fromLatin1("file:///")
+        << QString::fromLatin1("file:///")
+        << QString::fromLatin1("file:///");
+    QTest::newRow("local file 2")
+        << KUrl("file:///home/kde/")
+        << QString::fromLatin1("file:///home/kde/")
+        << QString::fromLatin1("file:///home/kde")
+        << QString::fromLatin1("file:///home/kde/");
+    QTest::newRow("local file 3")
+        << KUrl("file:///home/kde//")
+        << QString::fromLatin1("file:///home/kde//")
+        << QString::fromLatin1("file:///home/kde")
+        << QString::fromLatin1("file:///home/kde//");
+
+    QTest::newRow("ftp url")
+        << KUrl("ftp://ftp.kde.org/")
+        << QString::fromLatin1("ftp://ftp.kde.org/")
+        << QString::fromLatin1("ftp://ftp.kde.org/")
+        << QString::fromLatin1("ftp://ftp.kde.org/");
+    QTest::newRow("ftp url - 3 trailing slashes")
+        << KUrl("ftp://ftp.kde.org///")
+        << QString::fromLatin1("ftp://ftp.kde.org///")
+        << QString::fromLatin1("ftp://ftp.kde.org/")
+        << QString::fromLatin1("ftp://ftp.kde.org///");
+}
+
+void KUrlTest::testUrl()
+{
+    QFETCH( KUrl, url );
+    QFETCH( QString, urlLTS );
+    QFETCH( QString, urlRTS );
+    QFETCH( QString, urlATS );
+
+    QCOMPARE( url.url(KUrl::LeaveTrailingSlash), urlLTS );
+    QCOMPARE( url.url(KUrl::RemoveTrailingSlash), urlRTS );
+    QCOMPARE( url.url(KUrl::AddTrailingSlash), urlATS );
+}
+
+void KUrlTest::testToStringList()
+{
+    KUrl::List urls;
+    urls << KUrl("file:///")
+         << KUrl("file:///home/kde/")
+         << KUrl("file:///home/kde//")
+         << KUrl("ftp://ftp.kde.org/")
+         << KUrl("ftp://ftp.kde.org///");
+
+    //kDebug() << urls.toStringList(KUrl::LeaveTrailingSlash);
+    QCOMPARE( urls.toStringList(KUrl::LeaveTrailingSlash),
+              QStringList()
+              << QLatin1String("file:///")
+              << QLatin1String("file:///home/kde/")
+              << QLatin1String("file:///home/kde//")
+              << QLatin1String("ftp://ftp.kde.org/")
+              << QLatin1String("ftp://ftp.kde.org///") );
+
+    //kDebug() << urls.toStringList(KUrl::RemoveTrailingSlash);
+    QCOMPARE( urls.toStringList(KUrl::RemoveTrailingSlash),
+              QStringList()
+              << QLatin1String("file:///")
+              << QLatin1String("file:///home/kde")
+              << QLatin1String("file:///home/kde")
+              << QLatin1String("ftp://ftp.kde.org/")
+              << QLatin1String("ftp://ftp.kde.org/") );
+
+    //kDebug() << urls.toStringList(KUrl::AddTrailingSlash);
+    QCOMPARE( urls.toStringList(KUrl::AddTrailingSlash),
+              QStringList()
+              << QLatin1String("file:///")
+              << QLatin1String("file:///home/kde/")
+              << QLatin1String("file:///home/kde//")
+              << QLatin1String("ftp://ftp.kde.org/")
+              << QLatin1String("ftp://ftp.kde.org///") );
+}
