@@ -23,41 +23,48 @@
  */
 
 #include "kiconloader.h"
-#include "kicontheme.h"
-#include "kiconeffect.h"
-#include "kiconcache.h"
-#include "k3icon_p.h"
-
-#include <QtCore/QCache>
-#include <QtGui/QPixmap>
-#include <QtGui/QPixmapCache>
-#include <QtGui/QImage>
-#include <QtCore/QFileInfo>
-#include <QtCore/QDir>
-#include <QtCore/QBuffer>
-#include <QtCore/QDataStream>
-#include <QtCore/QByteArray>
-#include <QtGui/QIcon>
-#include <QtGui/QPainter>
-#include <QMovie>
-
-#include <kapplication.h>
-#include <kconfig.h>
-#include <kdebug.h>
-#include <kstandarddirs.h>
-#include <kglobal.h>
-#include <kglobalsettings.h>
-#include <kcomponentdata.h>
-#include <ksvgrenderer.h>
-#include <kde_file.h>
-#include <kshareddatacache.h>
 
 #include <sys/types.h>
 #include <stdlib.h>     //for abs
 #include <unistd.h>     //for readlink
 #include <dirent.h>
 #include <assert.h>
+
+#include <QtCore/QCache>
+#include <QtCore/QFileInfo>
+#include <QtCore/QDir>
+#include <QtCore/QBuffer>
+#include <QtCore/QDataStream>
+#include <QtCore/QByteArray>
+#include <QtCore/QStringBuilder> // % operator for QString
+#include <QtGui/QIcon>
+#include <QtGui/QImage>
+#include <QtGui/QMovie>
+#include <QtGui/QPainter>
+#include <QtGui/QPixmap>
+#include <QtGui/QPixmapCache>
+#ifndef _WIN32_WCE
+#include <QtSvg/QSvgRenderer>
+#endif
+
+// kdecore
+#include <kconfig.h>
 #include <kconfiggroup.h>
+#include <kdebug.h>
+#include <kstandarddirs.h>
+#include <kglobal.h>
+#include <kglobalsettings.h>
+#include <kcomponentdata.h>
+#include <kde_file.h>
+#include <kshareddatacache.h>
+
+// kdeui
+#include "kicontheme.h"
+#include "kiconeffect.h"
+#include "k3icon_p.h"
+
+// Used to make cache keys for icons with no group. Result type is QString*
+K_GLOBAL_STATIC_WITH_ARGS(QString, NULL_EFFECT_FINGERPRINT, (QString::fromLatin1("noeffect")))
 
 // Qt implements Tiny SVG specification. This specification does not cover important elements
 // that are pretty globally used on our icons, like blurring (and other filters). TT seems to have
@@ -66,8 +73,6 @@
 // (ereslibre)
 #undef KDE_QT_SVG_RENDERER_FIXED
 
-//#define NO_LAZYLOAD_ICONTHEME
-
 /**
  * Checks for relative paths quickly on UNIX-alikes, slowly on everything else.
  */
@@ -162,7 +167,6 @@ public:
     KIconLoaderPrivate(KIconLoader *q)
         : q(q)
         , mpGroups(0)
-        , mIconThemeCache(0)
         , mIconCache(0)
     {
     }
@@ -173,7 +177,6 @@ public:
         deleted when the elements of d->links are deleted */
         qDeleteAll(links);
         delete[] mpGroups;
-        delete mIconThemeCache;
         delete mIconCache;
     }
 
@@ -287,7 +290,6 @@ public:
     KStandardDirs *mpDirs;
     KIconEffect mpEffect;
     QList<KIconThemeNode *> links;
-    KIconCache* mIconThemeCache;
 
     // This shares the icons across all processes
     KSharedDataCache* mIconCache;
@@ -358,7 +360,9 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
         return;
     }
 
-    const int iconSize = pix.size().width();
+    const int width = pix.size().width();
+    const int height = pix.size().height();
+    const int iconSize = qMin(width, height);
     int overlaySize;
 
     if (iconSize < 32) {
@@ -398,16 +402,16 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
         switch (count) {
         case 0:
             // bottom left corner
-            startPoint = QPoint(2, iconSize - overlaySize - 2);
+            startPoint = QPoint(2, height - overlaySize - 2);
             break;
         case 1:
             // bottom right corner
-            startPoint = QPoint(iconSize - overlaySize - 2,
-                                iconSize - overlaySize - 2);
+            startPoint = QPoint(width - overlaySize - 2,
+                                height - overlaySize - 2);
             break;
         case 2:
             // top right corner
-            startPoint = QPoint(iconSize - overlaySize - 2, 2);
+            startPoint = QPoint(width - overlaySize - 2, 2);
             break;
         case 3:
             // top left corner
@@ -474,42 +478,31 @@ void KIconLoaderPrivate::init( const QString& _appname, KStandardDirs *_dirs )
     // smaller.
     mPixmapCache.setMaxCost(10 * 1024 * 1024);
 
-    // Initialize icon theme cache
-    mIconThemeCache = new KIconCache;
-    if (!mIconThemeCache->isValid()) {
-        initIconThemes();
-        QList<KIconTheme*> allThemes;
-        foreach (KIconThemeNode* node, links) {
-            allThemes.append(node->theme);
-        }
-        mIconThemeCache->setThemeInfo(allThemes);
-    }
-
     // These have to match the order in kicontheme.h
     static const char * const groups[] = { "Desktop", "Toolbar", "MainToolbar", "Small", "Panel", "Dialog", 0L };
     KSharedConfig::Ptr config = KGlobal::config();
 
     // loading config and default sizes
+    initIconThemes();
+    KIconTheme *defaultSizesTheme = links.empty() ? 0 : links.first()->theme;
     mpGroups = new KIconGroup[(int) KIconLoader::LastGroup];
-    for (KIconLoader::Group i=KIconLoader::FirstGroup; i<KIconLoader::LastGroup; i++)
-    {
-        if (groups[i] == 0L)
+    for (KIconLoader::Group i = KIconLoader::FirstGroup; i < KIconLoader::LastGroup; ++i) {
+        if (groups[i] == 0L) {
             break;
+        }
 
         KConfigGroup cg(config, QLatin1String(groups[i]) + "Icons");
         mpGroups[i].size = cg.readEntry("Size", 0);
-        if (QPixmap::defaultDepth()>8)
+        if (QPixmap::defaultDepth() > 8) {
             mpGroups[i].alphaBlending = cg.readEntry("AlphaBlending", true);
-        else
+        } else {
             mpGroups[i].alphaBlending = false;
+        }
 
-        if (!mpGroups[i].size)
-            mpGroups[i].size = mIconThemeCache->defaultIconSize(i);
+        if (!mpGroups[i].size && defaultSizesTheme) {
+            mpGroups[i].size = defaultSizesTheme->defaultSize(i);
+        }
     }
-
-#ifdef NO_LAZYLOAD_ICONTHEME
-    initIconThemes();
-#endif
 }
 
 bool KIconLoaderPrivate::initIconThemes()
@@ -709,6 +702,11 @@ bool KIconLoader::extraDesktopThemesAdded() const
     return d->extraDesktopIconsLoaded;
 }
 
+void KIconLoader::drawOverlays(const QStringList &overlays, QPixmap &pixmap, KIconLoader::Group group, int state) const
+{
+    d->drawOverlays(this, group, state, pixmap, overlays);
+}
+
 QString KIconLoaderPrivate::removeIconExtension(const QString &name) const
 {
     if (name.endsWith(QLatin1String(".png"))
@@ -761,26 +759,19 @@ void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &s
 QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group group,
                                          const QStringList &overlays, int size, int state) const
 {
-    // The KSharedDataCache is shared so add some namespacing.
-    QString key;
+    // The KSharedDataCache is shared so add some namespacing. The following code
+    // uses QStringBuilder (new in Qt 4.6)
 
-    if (group == KIconLoader::User) {
-        key = QLatin1String("$kicou_");
-    }
-    else {
-        key = QLatin1String("$kico_");
-    }
-
-    key.append(QString("%1_%2_%3").arg(name).arg(size).arg(overlays.join("_")));
-
-    if (group >= 0) {
-        key.append(mpEffect.fingerprint(group, state));
-    }
-    else {
-        key.append(QLatin1String("noeffect"));
-    }
-
-    return key;
+    return (group == KIconLoader::User
+                  ? QLatin1Literal("$kicou_")
+                  : QLatin1Literal("$kico_"))
+           % name
+           % QLatin1Char('_')
+           % QString::number(size)
+           % QLatin1Char('_')
+           % overlays.join("_")
+           % ( group >= 0 ? mpEffect.fingerprint(group, state)
+                          : *NULL_EFFECT_FINGERPRINT);
 }
 
 QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
@@ -801,7 +792,8 @@ QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
     }
     else
     {
-        KSvgRenderer renderer(path, q);
+#ifndef _WIN32_WCE
+        QSvgRenderer renderer(path, q);
 
         if (renderer.isValid()) {
             img = QImage(size, size, QImage::Format_ARGB32_Premultiplied);
@@ -809,6 +801,7 @@ QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
             QPainter p(&img);
             renderer.render(&p);
         }
+#endif
     }
 
     return img;
@@ -1184,7 +1177,6 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
                               QString *path_store, bool canReturnNull) const
 {
     QString name = _name;
-    bool unknownIcon = false;
     bool favIconOverlay = false;
 
     if (size < 0 || _name.isEmpty())
@@ -1312,7 +1304,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     // Don't add the path to our unknown icon to the cache, only cache the
     // actual image.
     if (iconWasUnknown) {
-        icon.path = QString();
+        icon.path.clear();
     }
 
     d->insertCachedPixmapWithPath(key, pix, icon.path);
@@ -1589,6 +1581,7 @@ bool KIconLoader::alphaBlending(KIconLoader::Group group) const
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon KIconLoader::loadIconSet( const QString& name, KIconLoader::Group g, int s,
                                 bool canReturnNull )
 {
@@ -1602,6 +1595,7 @@ QIcon KIconLoader::loadIconSet( const QString& name, KIconLoader::Group g, int s
     iconset.addPixmap( tmp, QIcon::Normal, QIcon::On );
     return iconset;
 }
+#endif
 
 // Easy access functions
 
@@ -1612,11 +1606,13 @@ QPixmap DesktopIcon(const QString& name, int force_size, int state, const QStrin
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon DesktopIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Desktop, force_size);
 }
+#endif
 
 QPixmap BarIcon(const QString& name, int force_size, int state, const QStringList &overlays)
 {
@@ -1625,11 +1621,13 @@ QPixmap BarIcon(const QString& name, int force_size, int state, const QStringLis
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon BarIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::Toolbar, force_size );
 }
+#endif
 
 QPixmap SmallIcon(const QString& name, int force_size, int state, const QStringList &overlays)
 {
@@ -1638,11 +1636,13 @@ QPixmap SmallIcon(const QString& name, int force_size, int state, const QStringL
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon SmallIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::Small, force_size );
 }
+#endif
 
 QPixmap MainBarIcon(const QString& name, int force_size, int state, const QStringList &overlays)
 {
@@ -1651,11 +1651,13 @@ QPixmap MainBarIcon(const QString& name, int force_size, int state, const QStrin
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon MainBarIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::MainToolbar, force_size );
 }
+#endif
 
 QPixmap UserIcon(const QString& name, int state, const QStringList &overlays)
 {
@@ -1664,11 +1666,13 @@ QPixmap UserIcon(const QString& name, int state, const QStringList &overlays)
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon UserIconSet(const QString& name)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::User );
 }
+#endif
 
 int IconSize(KIconLoader::Group group)
 {
