@@ -54,6 +54,7 @@
 #include <krecentdirs.h>
 #include <kdebug.h>
 #include <kio/kfileitemdelegate.h>
+#include <kde_file.h>
 
 #include <QtGui/QCheckBox>
 #include <QtGui/QDockWidget>
@@ -186,7 +187,6 @@ public:
     void _k_zoomInIconsSize();
     void _k_slotIconSizeSliderMoved(int);
     void _k_slotIconSizeChanged(int);
-    void _k_slotViewDoubleClicked(const QModelIndex&);
 
     void addToRecentDocuments();
 
@@ -197,7 +197,7 @@ public:
      * This method won't show any progress dialogs for stating, since
      * they are very annoying when stating.
      */
-    static KUrl mostLocalUrl(const KUrl &url);
+    KUrl mostLocalUrl(const KUrl &url);
 
     void setInlinePreviewShown(bool show);
 
@@ -254,6 +254,7 @@ public:
     KUrlComboBox *locationEdit;
     KDirOperator *ops;
     KFileFilterCombo *filterWidget;
+    QTimer filterDelayTimer;
 
     KFilePlacesModel *model;
 
@@ -392,12 +393,12 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     d->ops->setObjectName( "KFileWidget::ops" );
     d->ops->setIsSaving(d->operationMode == Saving);
     opsWidgetLayout->addWidget(d->ops);
-    connect(d->ops, SIGNAL(urlEntered(const KUrl&)),
-            SLOT(_k_urlEntered(const KUrl&)));
-    connect(d->ops, SIGNAL(fileHighlighted(const KFileItem &)),
-            SLOT(_k_fileHighlighted(const KFileItem &)));
-    connect(d->ops, SIGNAL(fileSelected(const KFileItem &)),
-            SLOT(_k_fileSelected(const KFileItem &)));
+    connect(d->ops, SIGNAL(urlEntered(KUrl)),
+            SLOT(_k_urlEntered(KUrl)));
+    connect(d->ops, SIGNAL(fileHighlighted(KFileItem)),
+            SLOT(_k_fileHighlighted(KFileItem)));
+    connect(d->ops, SIGNAL(fileSelected(KFileItem)),
+            SLOT(_k_fileSelected(KFileItem)));
     connect(d->ops, SIGNAL(finishedLoading()),
             SLOT(_k_slotLoadingFinished()));
 
@@ -426,21 +427,21 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     coll->action( "mkdir" )->setShortcut( QKeySequence(Qt::Key_F10) );
     coll->action( "mkdir" )->setWhatsThis(i18n("Click this button to create a new folder."));
 
-    KAction *goToNavigatorAction = coll->addAction( "gotonavigator", this, SLOT( _k_activateUrlNavigator() ) );
+    KAction *goToNavigatorAction = coll->addAction( "gotonavigator", this, SLOT(_k_activateUrlNavigator()) );
     goToNavigatorAction->setShortcut( QKeySequence(Qt::CTRL + Qt::Key_L) );
 
     KToggleAction *showSidebarAction =
         new KToggleAction(i18n("Show Places Navigation Panel"), this);
     coll->addAction("toggleSpeedbar", showSidebarAction);
     showSidebarAction->setShortcut( QKeySequence(Qt::Key_F9) );
-    connect( showSidebarAction, SIGNAL( toggled( bool ) ),
-             SLOT( _k_toggleSpeedbar( bool )) );
+    connect( showSidebarAction, SIGNAL(toggled(bool)),
+             SLOT(_k_toggleSpeedbar(bool)) );
 
     KToggleAction *showBookmarksAction =
         new KToggleAction(i18n("Show Bookmarks"), this);
     coll->addAction("toggleBookmarks", showBookmarksAction);
-    connect( showBookmarksAction, SIGNAL( toggled( bool ) ),
-             SLOT( _k_toggleBookmarks( bool )) );
+    connect( showBookmarksAction, SIGNAL(toggled(bool)),
+             SLOT(_k_toggleBookmarks(bool)) );
 
     KActionMenu *menu = new KActionMenu( KIcon("configure"), i18n("Options"), this);
     coll->addAction("extra menu", menu);
@@ -469,8 +470,8 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     menu->addAction( coll->action( "preview" ));
 
     menu->setDelayed( false );
-    connect( menu->menu(), SIGNAL( aboutToShow() ),
-             d->ops, SLOT( updateSelectionDependentActions() ));
+    connect( menu->menu(), SIGNAL(aboutToShow()),
+             d->ops, SLOT(updateSelectionDependentActions()));
 
     d->iconSizeSlider = new QSlider(this);
     d->iconSizeSlider->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
@@ -487,9 +488,9 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     connect(d->ops, SIGNAL(currentIconSizeChanged(int)),
             d->iconSizeSlider, SLOT(setValue(int)));
 
-    KAction *furtherAction = new KAction(KIcon("zoom-out"), i18n("Zoom out"), this);
+    KAction *furtherAction = new KAction(KIcon("file-zoom-out"), i18n("Zoom out"), this);
     connect(furtherAction, SIGNAL(triggered()), SLOT(_k_zoomOutIconsSize()));
-    KAction *closerAction = new KAction(KIcon("zoom-in"), i18n("Zoom in"), this);
+    KAction *closerAction = new KAction(KIcon("file-zoom-in"), i18n("Zoom in"), this);
     connect(closerAction, SIGNAL(triggered()), SLOT(_k_zoomInIconsSize()));
 
     QWidget *midSpacer = new QWidget(this);
@@ -522,10 +523,10 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     pathCombo->setCompletionObject( pathCompletionObj );
     pathCombo->setAutoDeleteCompletionObject( true );
 
-    connect( d->urlNavigator, SIGNAL( urlChanged( const KUrl&  )),
-             this,  SLOT( _k_enterUrl( const KUrl& ) ));
-    connect( d->urlNavigator, SIGNAL( returnPressed() ),
-             d->ops,  SLOT( setFocus() ));
+    connect( d->urlNavigator, SIGNAL(urlChanged(KUrl)),
+             this,  SLOT(_k_enterUrl(KUrl)));
+    connect( d->urlNavigator, SIGNAL(returnPressed()),
+             d->ops,  SLOT(setFocus()));
 
     QString whatsThisText;
 
@@ -537,8 +538,8 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     // huge dialogs that can't be resized to smaller (it would be as big as the longest
     // item in this combo box). (ereslibre)
     d->locationEdit->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLength);
-    connect( d->locationEdit, SIGNAL( editTextChanged( const QString& ) ),
-             SLOT( _k_slotLocationChanged( const QString& )) );
+    connect( d->locationEdit, SIGNAL(editTextChanged(QString)),
+             SLOT(_k_slotLocationChanged(QString)) );
 
     d->updateLocationWhatsThis();
     d->locationLabel->setBuddy(d->locationEdit);
@@ -546,11 +547,11 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     KUrlCompletion *fileCompletionObj = new KUrlCompletion( KUrlCompletion::FileCompletion );
     d->locationEdit->setCompletionObject( fileCompletionObj );
     d->locationEdit->setAutoDeleteCompletionObject( true );
-    connect( fileCompletionObj, SIGNAL( match( const QString& ) ),
-             SLOT( _k_fileCompletion( const QString& )) );
+    connect( fileCompletionObj, SIGNAL(match(QString)),
+             SLOT(_k_fileCompletion(QString)) );
 
-    connect(d->locationEdit, SIGNAL( returnPressed( const QString&  )),
-            this,  SLOT( _k_locationAccepted( const QString& ) ));
+    connect(d->locationEdit, SIGNAL(returnPressed(QString)),
+            this,  SLOT(_k_locationAccepted(QString)));
 
     // the Filter label/edit
     whatsThisText = i18n("<qt>This is the filter to apply to the file list. "
@@ -570,6 +571,11 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     d->filterLabel->setBuddy(d->filterWidget);
     connect(d->filterWidget, SIGNAL(filterChanged()), SLOT(_k_slotFilterChanged()));
 
+    d->filterDelayTimer.setSingleShot(true);
+    d->filterDelayTimer.setInterval(300);
+    connect(d->filterWidget, SIGNAL(editTextChanged(QString)), &d->filterDelayTimer, SLOT(start()));
+    connect(&d->filterDelayTimer, SIGNAL(timeout()), SLOT(_k_slotFilterChanged()));
+
     // the Automatically Select Extension checkbox
     // (the text, visibility etc. is set in updateAutoSelectExtension(), which is called by readConfig())
     d->autoSelectExtCheckBox = new QCheckBox (this);
@@ -602,7 +608,7 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     if ( filename.isEmpty() )
     {
         KIO::StatJob *statJob = KIO::stat(startDir, KIO::HideProgressInfo);
-        statRes = KIO::NetAccess::synchronousRun(statJob, 0);
+        statRes = KIO::NetAccess::synchronousRun(statJob, this);
         kDebug(kfile_area) << "stat of" << startDir << "-> statRes" << statRes << "isDir" << statJob->statResult().isDir();
         if (!statRes || !statJob->statResult().isDir()) {
             filename = startDir.fileName();
@@ -818,7 +824,7 @@ void KFileWidget::slotOk()
             while (!res && start < locationEditCurrentTextList.count()) {
                 topMostUrl = locationEditCurrentTextList.at(start);
                 statJob = KIO::stat(topMostUrl, KIO::HideProgressInfo);
-                res = KIO::NetAccess::synchronousRun(statJob, 0);
+                res = KIO::NetAccess::synchronousRun(statJob, this);
                 start++;
             }
 
@@ -835,7 +841,7 @@ void KFileWidget::slotOk()
             for (int i = start; i < locationEditCurrentTextList.count(); ++i) {
                 KUrl currUrl = locationEditCurrentTextList.at(i);
                 KIO::StatJob *statJob = KIO::stat(currUrl, KIO::HideProgressInfo);
-                bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+                bool res = KIO::NetAccess::synchronousRun(statJob, this);
                 if (res) {
                     // again, we don't care about filenames
                     if (!statJob->statResult().isDir()) {
@@ -881,7 +887,7 @@ void KFileWidget::slotOk()
             KUrl url(locationEditCurrentText);
             if (d->operationMode == Opening) {
                 KIO::StatJob *statJob = KIO::stat(url, KIO::HideProgressInfo);
-                bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+                bool res = KIO::NetAccess::synchronousRun(statJob, this);
                 if (res) {
                     if (!statJob->statResult().isDir()) {
                         url.adjustPath(KUrl::RemoveTrailingSlash);
@@ -896,7 +902,7 @@ void KFileWidget::slotOk()
                 directory.setFileName(QString());
                 //Check if the folder exists
                 KIO::StatJob * statJob = KIO::stat(directory, KIO::HideProgressInfo);
-                bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+                bool res = KIO::NetAccess::synchronousRun(statJob, this);
                 if (res) {
                     if (statJob->statResult().isDir()) {
                         url.adjustPath(KUrl::RemoveTrailingSlash);
@@ -933,7 +939,7 @@ void KFileWidget::slotOk()
 
         d->url = url;
         KIO::StatJob *statJob = KIO::stat(url, KIO::HideProgressInfo);
-        bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+        bool res = KIO::NetAccess::synchronousRun(statJob, this);
 
         if (!KAuthorized::authorizeUrlAction("open", KUrl(), url)) {
             QString msg = KIO::buildErrorString(KIO::ERR_ACCESS_DENIED, d->url.prettyUrl());
@@ -961,7 +967,7 @@ void KFileWidget::slotOk()
             while (it != locationEditCurrentTextList.constEnd()) {
                 KUrl checkUrl(*it);
                 KIO::StatJob *checkStatJob = KIO::stat(checkUrl, KIO::HideProgressInfo);
-                bool res = KIO::NetAccess::synchronousRun(checkStatJob, 0);
+                bool res = KIO::NetAccess::synchronousRun(checkStatJob, this);
                 if (res && checkStatJob->statResult().isDir()) {
                     KMessageBox::sorry(this, i18n("More than one folder has been selected and this dialog does not accept folders, so it is not possible to decide which one to enter. Please select only one folder to list it."), i18n("More than one folder provided"));
                     return;
@@ -1137,8 +1143,8 @@ void KFileWidgetPrivate::setDummyHistoryEntry( const QString& text, const QPixma
     // setCurrentItem() will cause textChanged() being emitted,
     // so slotLocationChanged() will be called. Make sure we don't clear
     // the KDirOperator's view-selection in there
-    QObject::disconnect( locationEdit, SIGNAL( editTextChanged( const QString& ) ),
-                        q, SLOT( _k_slotLocationChanged( const QString& ) ) );
+    QObject::disconnect( locationEdit, SIGNAL(editTextChanged(QString)),
+                        q, SLOT(_k_slotLocationChanged(QString)) );
 
     bool dummyExists = dummyAdded;
 
@@ -1176,8 +1182,8 @@ void KFileWidgetPrivate::setDummyHistoryEntry( const QString& text, const QPixma
 
     locationEdit->lineEdit()->setCursorPosition( cursorPosition );
 
-    QObject::connect( locationEdit, SIGNAL( editTextChanged ( const QString& ) ),
-                    q, SLOT( _k_slotLocationChanged( const QString& )) );
+    QObject::connect( locationEdit, SIGNAL(editTextChanged(QString)),
+                    q, SLOT(_k_slotLocationChanged(QString)) );
 }
 
 void KFileWidgetPrivate::removeDummyHistoryEntry()
@@ -1189,8 +1195,8 @@ void KFileWidgetPrivate::removeDummyHistoryEntry()
     // setCurrentItem() will cause textChanged() being emitted,
     // so slotLocationChanged() will be called. Make sure we don't clear
     // the KDirOperator's view-selection in there
-    QObject::disconnect( locationEdit, SIGNAL( editTextChanged( const QString& ) ),
-                        q, SLOT( _k_slotLocationChanged( const QString& ) ) );
+    QObject::disconnect( locationEdit, SIGNAL(editTextChanged(QString)),
+                        q, SLOT(_k_slotLocationChanged(QString)) );
 
     if (locationEdit->count()) {
         locationEdit->removeItem( 0 );
@@ -1198,8 +1204,8 @@ void KFileWidgetPrivate::removeDummyHistoryEntry()
     locationEdit->setCurrentIndex( -1 );
     dummyAdded = false;
 
-    QObject::connect( locationEdit, SIGNAL( editTextChanged ( const QString& ) ),
-                    q, SLOT( _k_slotLocationChanged( const QString& )) );
+    QObject::connect( locationEdit, SIGNAL(editTextChanged(QString)),
+                    q, SLOT(_k_slotLocationChanged(QString)) );
 }
 
 void KFileWidgetPrivate::setLocationText(const KUrl& url)
@@ -1371,16 +1377,22 @@ void KFileWidgetPrivate::_k_slotFilterChanged()
 {
 //     kDebug(kfile_area);
 
+    filterDelayTimer.stop();
+
     QString filter = filterWidget->currentFilter();
     ops->clearFilter();
 
-    if ( filter.indexOf( '/' ) > -1 ) {
+    if ( filter.contains('/') ) {
         QStringList types = filter.split(' ', QString::SkipEmptyParts);
         types.prepend("inode/directory");
         ops->setMimeFilter( types );
     }
-    else
+    else if ( filter.contains('*') || filter.contains('?') || filter.contains('[') ) {
         ops->setNameFilter( filter );
+    }
+    else {
+        ops->setNameFilter('*' + filter.replace(' ', '*') + '*');
+    }
 
     ops->updateDir();
 
@@ -1462,7 +1474,7 @@ bool KFileWidgetPrivate::toOverwrite(const KUrl &url)
 //     kDebug(kfile_area);
 
     KIO::StatJob *statJob = KIO::stat(url, KIO::HideProgressInfo);
-    bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+    bool res = KIO::NetAccess::synchronousRun(statJob, q);
 
     if (res) {
         int ret = KMessageBox::warningContinueCancel( q,
@@ -1741,8 +1753,6 @@ void KFileWidget::showEvent(QShowEvent* event)
         d->ops->setView( KFile::Default );
         d->ops->view()->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
         d->hasView = true;
-
-        connect(d->ops->view(), SIGNAL(doubleClicked(QModelIndex)), this, SLOT(_k_slotViewDoubleClicked(QModelIndex)));
     }
     d->ops->clearHistory();
 
@@ -1865,29 +1875,29 @@ void KFileWidgetPrivate::writeConfig(KConfigGroup &configGroup)
 
     // these settings are global settings; ALL instances of the file dialog
     // should reflect them
-    configGroup.config()->setForceGlobal(true);
+    KConfig config("kdeglobals");
+    KConfigGroup group(&config, configGroup.name());
 
     KUrlComboBox *pathCombo = urlNavigator->editor();
-    configGroup.writePathEntry( RecentURLs, pathCombo->urls() );
-    //saveDialogSize( configGroup, KConfigGroup::Persistent | KConfigGroup::Global );
-    configGroup.writeEntry( PathComboCompletionMode, static_cast<int>(pathCombo->completionMode()) );
-    configGroup.writeEntry( LocationComboCompletionMode, static_cast<int>(locationEdit->completionMode()) );
+    group.writePathEntry( RecentURLs, pathCombo->urls() );
+    //saveDialogSize( group, KConfigGroup::Persistent | KConfigGroup::Global );
+    group.writeEntry( PathComboCompletionMode, static_cast<int>(pathCombo->completionMode()) );
+    group.writeEntry( LocationComboCompletionMode, static_cast<int>(locationEdit->completionMode()) );
 
     const bool showSpeedbar = placesDock && !placesDock->isHidden();
-    configGroup.writeEntry( ShowSpeedbar, showSpeedbar );
+    group.writeEntry( ShowSpeedbar, showSpeedbar );
     if (showSpeedbar) {
         const QList<int> sizes = placesViewSplitter->sizes();
         Q_ASSERT( sizes.count() > 0 );
-        configGroup.writeEntry( SpeedbarWidth, sizes[0] );
+        group.writeEntry( SpeedbarWidth, sizes[0] );
     }
 
-    configGroup.writeEntry( ShowBookmarks, bookmarkHandler != 0 );
-    configGroup.writeEntry( AutoSelectExtChecked, autoSelectExtChecked );
-    configGroup.writeEntry( BreadcrumbNavigation, !urlNavigator->isUrlEditable() );
-    configGroup.writeEntry( ShowFullPath, urlNavigator->showFullPath() );
+    group.writeEntry( ShowBookmarks, bookmarkHandler != 0 );
+    group.writeEntry( AutoSelectExtChecked, autoSelectExtChecked );
+    group.writeEntry( BreadcrumbNavigation, !urlNavigator->isUrlEditable() );
+    group.writeEntry( ShowFullPath, urlNavigator->showFullPath() );
 
-    ops->writeConfig(configGroup);
-    configGroup.config()->setForceGlobal(false);
+    ops->writeConfig(group);
 }
 
 
@@ -2058,14 +2068,6 @@ void KFileWidgetPrivate::_k_slotIconSizeSliderMoved(int _value)
     QApplication::sendEvent(iconSizeSlider, &toolTipEvent);
 }
 
-void KFileWidgetPrivate::_k_slotViewDoubleClicked(const QModelIndex &index)
-{
-    if (!index.isValid()) {
-        return;
-    }
-    q->slotOk();
-}
-
 static QString getExtensionFromPatternList(const QStringList &patternList)
 {
 //     kDebug(kfile_area);
@@ -2140,12 +2142,23 @@ void KFileWidgetPrivate::updateAutoSelectExtension()
         QString filter = filterWidget->currentFilter();
         if (!filter.isEmpty())
         {
+            // if the currently selected filename already has an extension which
+            // is also included in the currently allowed extensions, keep it
+            // otherwise use the default extension
+            QString currentExtension = KMimeType::extractKnownExtension(locationEditCurrentText());
+            if ( currentExtension.isEmpty() )
+                currentExtension = locationEditCurrentText().section(QLatin1Char('.'), -1, -1);
+            kDebug (kfile_area) << "filter:" << filter << "locationEdit:" << locationEditCurrentText()
+                                << "currentExtension:" << currentExtension;
+
+            QString defaultExtension;
+            QStringList extensionList;
+
             // e.g. "*.cpp"
             if (filter.indexOf ('/') < 0)
             {
-                extension = getExtensionFromPatternList (filter.split(' ', QString::SkipEmptyParts)/*QStringList::split (" ", filter)*/).toLower();
-//                 kDebug (kfile_area) << "\tsetFilter-style: pattern ext=\'"
-//                                     << extension << "\'" << endl;
+                extensionList = filter.split(' ', QString::SkipEmptyParts);
+                defaultExtension = getExtensionFromPatternList(extensionList);
             }
             // e.g. "text/html"
             else
@@ -2153,11 +2166,17 @@ void KFileWidgetPrivate::updateAutoSelectExtension()
                 KMimeType::Ptr mime = KMimeType::mimeType (filter);
                 if (mime)
                 {
-                        extension = mime->mainExtension().toLower();
-//                         kDebug (kfile_area) << "\tsetMimeFilter-style: pattern ext=\'"
-//                                             << extension << "\'" << endl;
+                    extensionList = mime->patterns();
+                    defaultExtension = mime->mainExtension();
                 }
             }
+
+            if ( !currentExtension.isEmpty() && extensionList.contains(QLatin1String("*.") + currentExtension) )
+                extension = QLatin1Char('.') + currentExtension;
+            else
+                extension = defaultExtension;
+
+            kDebug (kfile_area) << "List:" << extensionList << "auto-selected extension:" << extension;
         }
 
 
@@ -2260,7 +2279,7 @@ void KFileWidgetPrivate::updateLocationEditExtension (const QString &lastExtensi
     {
         // exists?
         KIO::StatJob *statJob = KIO::stat(url, KIO::HideProgressInfo);
-        bool result = KIO::NetAccess::synchronousRun(statJob, 0);
+        bool result = KIO::NetAccess::synchronousRun(statJob, q);
         if (result)
         {
 //             kDebug (kfile_area) << "\tfile exists";
@@ -2305,15 +2324,30 @@ void KFileWidgetPrivate::updateFilter()
 //     kDebug(kfile_area);
 
     if ((operationMode == KFileWidget::Saving) && (ops->mode() & KFile::File) ) {
-        const QString urlStr = locationEditCurrentText();
+        QString urlStr = locationEditCurrentText();
         if (urlStr.isEmpty())
             return;
 
-        KMimeType::Ptr mime = KMimeType::findByPath(urlStr, 0, true);
-        if (mime && mime->name() != KMimeType::defaultMimeType()) {
-            if (filterWidget->currentFilter() != mime->name() &&
-                filterWidget->filters().indexOf(mime->name()) != -1)
-                filterWidget->setCurrentFilter(mime->name());
+        if( filterWidget->isMimeFilter()) {
+            KMimeType::Ptr mime = KMimeType::findByPath(urlStr, 0, true);
+            if (mime && mime->name() != KMimeType::defaultMimeType()) {
+                if (filterWidget->currentFilter() != mime->name() &&
+                    filterWidget->filters().indexOf(mime->name()) != -1)
+                    filterWidget->setCurrentFilter(mime->name());
+            }
+        } else {
+            QString filename = urlStr.mid( urlStr.lastIndexOf( KDIR_SEPARATOR ) + 1 ); // only filename
+            foreach( const QString& filter, filterWidget->filters()) {
+                QStringList patterns = filter.left( filter.indexOf( '|' )).split ( ' ', QString::SkipEmptyParts ); // '*.foo *.bar|Foo type' -> '*.foo', '*.bar'
+                foreach ( const QString& p, patterns ) {
+                    if( KMimeType::matchFileName( filename, p )) {
+                        if ( p != "*" ) { // never match the catch-all filter
+                            filterWidget->setCurrentFilter( filter );
+                        }
+                        break;
+                    }
+                }
+            }
         }
     }
 }
@@ -2344,7 +2378,7 @@ void KFileWidgetPrivate::appendExtension (KUrl &url)
 
     // exists?
     KIO::StatJob *statJob = KIO::stat(url, KIO::HideProgressInfo);
-    bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+    bool res = KIO::NetAccess::synchronousRun(statJob, q);
     if (res)
     {
 //         kDebug (kfile_area) << "\tfile exists - won't append extension";
@@ -2474,8 +2508,8 @@ void KFileWidgetPrivate::_k_toggleBookmarks(bool show)
         }
 
         bookmarkHandler = new KFileBookmarkHandler( q );
-        q->connect( bookmarkHandler, SIGNAL( openUrl( const QString& )),
-                    SLOT( _k_enterUrl( const QString& )));
+        q->connect( bookmarkHandler, SIGNAL(openUrl(QString)),
+                    SLOT(_k_enterUrl(QString)));
 
         bookmarkButton = new KActionMenu(KIcon("bookmarks"),i18n("Bookmarks"), q);
         bookmarkButton->setDelayed(false);
@@ -2560,8 +2594,10 @@ KUrl KFileWidget::getStartUrl( const KUrl& startDir,
             {
                 ret = startDir;				// will be checked by stat later
                 // If we won't be able to list it (e.g. http), then use default
-                if ( !KProtocolManager::supportsListing( ret ) )
+                if ( !KProtocolManager::supportsListing( ret ) ) {
                     useDefaultStartDir = true;
+                    fileName = startDir.fileName();
+                }
             }
             else					// file name only
             {
@@ -2698,7 +2734,7 @@ KUrl KFileWidgetPrivate::mostLocalUrl(const KUrl &url)
     }
 
     KIO::StatJob *statJob = KIO::stat(url, KIO::HideProgressInfo);
-    bool res = KIO::NetAccess::synchronousRun(statJob, 0);
+    bool res = KIO::NetAccess::synchronousRun(statJob, q);
 
     if (!res) {
         return url;
