@@ -107,8 +107,6 @@ IMPLEMENT_CONSTANT_TABLE(DOMNodeConstants,"DOMNodeConstants")
 # IE extensions
   contains	DOMNode::Contains		DontDelete|Function 1
   insertAdjacentHTML	DOMNode::InsertAdjacentHTML	DontDelete|Function 2
-# "DOM level 0" (from Gecko DOM reference; also in WinIE)
-  item          DOMNode::Item           DontDelete|Function 1
 @end
 */
 KJS_IMPLEMENT_PROTOFUNC(DOMNodeProtoFunc)
@@ -693,10 +691,6 @@ JSValue* DOMNodeProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, co
 
       return jsUndefined();
     }
-    case DOMNode::Item: {
-      SharedPtr<NodeListImpl> childNodes = node.childNodes();
-      return getDOMNode(exec, childNodes->item(static_cast<unsigned long>(args[0]->toNumber(exec))));
-    }
     case DOMNode::CompareDocumentPosition: {
        DOM::NodeImpl* other = toNode(args[0]);
        if (!other)
@@ -1137,7 +1131,7 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
     return getDOMEvent(exec,doc.createEvent(s, exception));
   case DOMDocument::GetOverrideStyle: {
     DOM::NodeImpl* arg0 = toNode(args[0]);
-    if (!arg0->isElementNode())
+    if (!arg0 || !arg0->isElementNode())
       return jsUndefined(); // throw exception?
     else
       return getDOMCSSStyleDeclaration(exec,doc.getOverrideStyle(static_cast<DOM::ElementImpl*>(arg0),args[1]->toString(exec).domString().implementation()));
@@ -2138,7 +2132,7 @@ DOMComment::DOMComment(ExecState *exec, DOM::CommentImpl* d)
     setPrototype(DOMCommentProto::self(exec));
  }
 
-JSValue* DOMCommentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
+JSValue* DOMCommentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List & /*args*/)
 {
   KJS_CHECK_THIS( KJS::DOMComment, thisObj );
   return jsUndefined();
