@@ -28,6 +28,7 @@
 #include "dom/dom_exception.h"
 #include "dom/dom_string.h"
 #include "misc/loader.h"
+#include "misc/translator.h"
 #include "html/html_documentimpl.h"
 #include "xml/dom2_eventsimpl.h"
 
@@ -40,12 +41,8 @@
 #include <QtCore/QHash>
 #include <kdebug.h>
 
-#ifdef APPLE_CHANGES
-#include "KWQLoader.h"
-#else
 #include <kio/netaccess.h>
 using KIO::NetAccess;
-#endif
 
 using namespace KJS;
 using namespace DOM;
@@ -227,10 +224,11 @@ void XMLHttpRequest::putValueProperty(ExecState *exec, int token, JSValue *value
 // Token according to RFC 2616
 static bool isValidFieldName(const QString& name)
 {
-    const QChar* c = name.constData();
     int l = name.length();
     if (l == 0)
         return false;
+    
+    const QChar* c = name.constData();
     for (int i = 0; i < l; ++i, ++c) {
         int u = c->unicode();
         if (u < 32 || u > 126)
@@ -251,15 +249,15 @@ static bool isValidFieldName(const QString& name)
 
 static bool isValidFieldValue(const QString& name)
 {
-    const QChar* c = name.constData();
     int l = name.length();
     if (l == 0)
         return true;
 
+    const QChar* c = name.constData();
     for (int i = 0; i < l; ++i, ++c) {
         int u = c->unicode();
-	if ( u == '\n' || u == '\r' )
-	  return false;
+        if ( u == '\n' || u == '\r' )
+           return false;
     }
 
     // ### what is invalid?
@@ -271,7 +269,7 @@ static bool canSetRequestHeader(const QString& name)
     static QSet<CaseInsensitiveString> forbiddenHeaders;
 
     if (forbiddenHeaders.isEmpty()) {
-	static const char* hdrs[] = {
+	static const char* const hdrs[] = {
 	    "accept-charset",
 	    "accept-encoding",
 	    "content-length",
@@ -317,6 +315,7 @@ XMLHttpRequest::XMLHttpRequest(ExecState *exec, DOM::DocumentImpl* d)
     onReadyStateChangeListener(0),
     onLoadListener(0),
     decoder(0),
+    binaryMode(false),
     response(QString::fromLatin1("")),
     createdDocument(false),
     aborted(false)
@@ -327,7 +326,7 @@ XMLHttpRequest::XMLHttpRequest(ExecState *exec, DOM::DocumentImpl* d)
 
 XMLHttpRequest::~XMLHttpRequest()
 {
-  if (job && method.toUpper() != QLatin1String("POST")) {
+  if (job && m_method != QLatin1String("POST")) {
         job->kill();
         job = 0;
   }
@@ -347,7 +346,7 @@ void XMLHttpRequest::changeState(XMLHttpRequestState newState)
   // exist anymore. Match that, though being paranoid about post
   // (And don't emit any events w/o a doc, if we're kept alive otherwise).
   if (!doc) {
-    if (job && method.toUpper() != QLatin1String("POST")) {
+    if (job && m_method != QLatin1String("POST")) {
       job->kill();
       job = 0;
     }
@@ -406,6 +405,23 @@ bool XMLHttpRequest::urlMatchesDocumentDomain(const KUrl& _url) const
   return false;
 }
 
+// Methods we're to recognize per the XHR spec (3.6.1, #3).
+// We map it to whether the method should be permitted or not (#4)
+static const IDTranslator<QByteArray, bool, const char*>::Info methodsTable[] = {
+    {"CONNECT", false},
+    {"DELETE", true},
+    {"GET", true},
+    {"HEAD", true},
+    {"OPTIONS", true},
+    {"POST", true},
+    {"PUT", true},
+    {"TRACE", false},
+    {"TRACK", false},
+    {0, false}
+};
+
+MAKE_TRANSLATOR(methodsLookup, QByteArray, bool, const char*, methodsTable)
+
 void XMLHttpRequest::open(const QString& _method, const KUrl& _url, bool _async, int& ec)
 {
   abort();
@@ -423,7 +439,24 @@ void XMLHttpRequest::open(const QString& _method, const KUrl& _url, bool _async,
     return;
   }
 
-  method = _method.toLower();
+  // ### potentially raise a SYNTAX_ERR
+
+  // Lookup if the method is well-known, and if so check if it's OK
+  QByteArray methodNormalized = _method.toUpper().toUtf8();
+  if (methodsLookup()->hasLeft(methodNormalized)) {
+      if (methodsLookup()->toRight(methodNormalized)) {
+          // OK, replace with the canonical version...
+          m_method = _method.toUpper();
+      } else {
+          // Scary stuff like CONNECT
+          ec = DOMException::SECURITY_ERR;
+          return;
+      }
+  } else {
+      // Unknown -> pass through unchanged
+      m_method = _method;
+  }
+
   url = _url;
   async = _async;
 
@@ -445,12 +478,14 @@ void XMLHttpRequest::send(const QString& _body, int& ec)
   if (!protocol.startsWith(QLatin1String("http")) &&
       !protocol.startsWith(QLatin1String("webdav")))
   {
-    ec = DOMException::INVALID_ACCESS_ERR;
-    abort();
-    return;
+      ec = DOMException::INVALID_ACCESS_ERR;
+      abort();
+      return;
   }
 
-  if (method == "post") {
+  // We need to use a POST-like setup even for non-post whenever we
+  // have a payload.
+  if (m_method == QLatin1String("POST") || !_body.isEmpty()) {
 
     // FIXME: determine post encoding correctly by looking in headers
     // for charset.
@@ -467,6 +502,9 @@ void XMLHttpRequest::send(const QString& _body, int& ec)
     job = KIO::get( url, KIO::NoReload, KIO::HideProgressInfo );
   }
 
+  // Regardless of job type, make sure the method is set
+  job->addMetaData("CustomHTTPMethod", m_method);
+
   if (!m_requestHeaders.isEmpty()) {
     QString rh;
     HTTPHeaderMap::ConstIterator begin = m_requestHeaders.constBegin();
@@ -515,34 +553,40 @@ void XMLHttpRequest::send(const QString& _body, int& ec)
     return;
   }
 
-  qObject->connect( job, SIGNAL( result( KJob* ) ),
-		    SLOT( slotFinished( KJob* ) ) );
+  qObject->connect( job, SIGNAL(result(KJob*)),
+		    SLOT(slotFinished(KJob*)) );
 #ifdef APPLE_CHANGES
-  qObject->connect( job, SIGNAL( data( KIO::Job*, const char*, int ) ),
-		    SLOT( slotData( KIO::Job*, const char*, int ) ) );
+  qObject->connect( job, SIGNAL(data(KIO::Job*,const char*,int)),
+		    SLOT(slotData(KIO::Job*,const char*,int)) );
 #else
-  qObject->connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
-		    SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
+  qObject->connect( job, SIGNAL(data(KIO::Job*,QByteArray)),
+		    SLOT(slotData(KIO::Job*,QByteArray)) );
 #endif
-  qObject->connect( job, SIGNAL(redirection(KIO::Job*, const KUrl& ) ),
-		    SLOT( slotRedirection(KIO::Job*, const KUrl&) ) );
+  qObject->connect( job, SIGNAL(redirection(KIO::Job*,KUrl)),
+		    SLOT(slotRedirection(KIO::Job*,KUrl)) );
 
 #ifdef APPLE_CHANGES
   KWQServeRequest(khtml::Cache::loader(), doc->docLoader(), job);
 #else
-  KIO::Scheduler::scheduleJob( job );
+  KIO::Scheduler::setJobPriority( job, 1 );
 #endif
 }
 
+void XMLHttpRequest::clearDecoder()
+{
+    delete decoder;
+    decoder = 0;
+    binaryMode = false;
+}
+
 void XMLHttpRequest::abort()
 {
   if (job) {
     job->kill();
     job = 0;
   }
-  delete decoder;
-  decoder = 0;
   aborted = true;
+  clearDecoder();
   changeState(XHRS_Uninitialized);
 }
 
@@ -746,8 +790,7 @@ void XMLHttpRequest::slotFinished(KJob *)
   job = 0;
   changeState(XHRS_Loaded);
 
-  delete decoder;
-  decoder = 0;
+  clearDecoder();
 }
 
 void XMLHttpRequest::slotRedirection(KIO::Job*, const KUrl& url)
@@ -792,7 +835,7 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
   int len = (int)_data.size();
 #endif
 
-  if ( decoder == NULL ) {
+  if ( !decoder && !binaryMode ) {
     if (!m_mimeTypeOverride.isEmpty())
         encoding = encodingFromContentType(m_mimeTypeOverride);
 
@@ -806,20 +849,28 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
       }
     }
 
-    decoder = new KEncodingDetector;
-
-    if (!encoding.isEmpty())
-      decoder->setEncoding(encoding.toLatin1().constData(), KEncodingDetector::EncodingFromHTTPHeader);
-    else
-      decoder->setEncoding("UTF-8", KEncodingDetector::DefaultEncoding);
+    if (encoding == QLatin1String("x-user-defined")) {
+      binaryMode = true;
+    } else {
+      decoder = new KEncodingDetector;
+      if (!encoding.isEmpty())
+        decoder->setEncoding(encoding.toLatin1().constData(), KEncodingDetector::EncodingFromHTTPHeader);
+      else
+        decoder->setEncoding("UTF-8", KEncodingDetector::DefaultEncoding);
+    }
   }
+
   if (len == 0)
     return;
 
   if (len == -1)
     len = strlen(data);
 
-  QString decoded = decoder->decodeWithBuffering(data, len);
+  QString decoded;
+  if (binaryMode)
+    decoded = QString::fromLatin1(data, len);
+  else
+    decoded = decoder->decodeWithBuffering(data, len);
 
   response += decoded;
 
@@ -867,10 +918,7 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
           return throwError(exec, SyntaxError, "Not enough arguments");
 
       QString method = args[0]->toString(exec).qstring();
-      KHTMLPart *part = qobject_cast<KHTMLPart*>(Window::retrieveActive(exec)->part());
-      if (!part)
-        return jsUndefined();
-      KUrl url = KUrl(part->document().completeURL(args[1]->toString(exec).qstring()).string());
+      KUrl url = KUrl(request->doc->completeURL(args[1]->toString(exec).qstring()));
 
       bool async = true;
       if (args.size() >= 3) {
@@ -892,21 +940,20 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
   case XMLHttpRequest::Send:
     {
       QString body;
-      if (args.size() >= 1) {
-        DOM::NodeImpl* docNode = toNode(args[0]);
-        if (docNode && docNode->isDocumentNode()) {
-          DOM::DocumentImpl *doc = static_cast<DOM::DocumentImpl *>(docNode);
+      if (!args[0]->isUndefinedOrNull()
+            // make sure we don't marshal "undefined" or such;
+          && request->m_method != QLatin1String("GET")
+          && request->m_method != QLatin1String("HEAD")) {
+            // ... or methods that don't have payload
 
-          try {
-            body = doc->toString().string();
-            // FIXME: also need to set content type, including encoding!
-
-          } catch(DOM::DOMException&) {
-            return throwError(exec, GeneralError, "Exception serializing document");
+          DOM::NodeImpl* docNode = toNode(args[0]);
+          if (docNode && docNode->isDocumentNode()) {
+              DOM::DocumentImpl *doc = static_cast<DOM::DocumentImpl *>(docNode);
+              body = doc->toString().string();
+              // FIXME: also need to set content type, including encoding!
+          } else {
+              body = args[0]->toString(exec).qstring();
           }
-        } else {
-          body = args[0]->toString(exec).qstring();
-        }
       }
 
       request->send(body, ec);
@@ -943,3 +990,4 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
 } // end namespace
 
 #include "xmlhttprequest.moc"
+// kate: indent-width 2; replace-tabs on; tab-width 4; space-indent on;
