@@ -53,9 +53,10 @@
 #include "misc/helper.h"
 #include "misc/paintbuffer.h"
 #include "css/cssvalues.h"
-#include "misc/htmltags.h"
 #include <kdebug.h>
 
+//#define IN_PLACE_WIDGET_PAINTING
+
 bool khtml::allowWidgetPaintEvents = false;
 
 using namespace khtml;
@@ -185,6 +186,7 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
     m_underMouse = 0;
     m_buffer[0] = 0;
     m_buffer[1] = 0;
+    m_nativeFrameShape = QFrame::NoFrame;
     // a widget doesn't support being anonymous
     assert(!isAnonymous());
     m_view  = node->document()->view();
@@ -292,7 +294,7 @@ void RenderWidget::setQWidget(QWidget *widget)
     {
         if (m_widget) {
             m_widget->removeEventFilter(this);
-            disconnect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
+            disconnect( m_widget, SIGNAL(destroyed()), this, SLOT(slotWidgetDestructed()));
             m_widget->hide();
             if (m_ownsWidget)
                 m_widget->deleteLater(); //Might happen due to event on the widget, so be careful
@@ -313,7 +315,7 @@ void RenderWidget::setQWidget(QWidget *widget)
             m_widget->setParent(m_view->widget());
             if (isRedirectedSubFrame)
                 static_cast<KHTMLView*>(m_widget)->setHasStaticBackground();
-            connect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
+            connect( m_widget, SIGNAL(destroyed()), this, SLOT(slotWidgetDestructed()));
             m_widget->installEventFilter(this);
             if (isRedirectedWidget()) {
                 if (!qobject_cast<QFrame*>(m_widget))
@@ -480,12 +482,18 @@ void RenderWidget::updateFromElement()
         }
 
         // Border:
-        if (shouldPaintBorder() || (!shouldPaintBackgroundOrBorder() && canHaveBorder()))
-        {
-            if (QFrame* frame = qobject_cast<QFrame*>(m_widget))
-                frame->setFrameShape(QFrame::NoFrame);
-            else if (QLineEdit* lineEdit = qobject_cast<QLineEdit*>(m_widget))
-                lineEdit->setFrame(false);
+        if (QFrame* frame = qobject_cast<QFrame*>(m_widget)) {
+            if (shouldDisableNativeBorders()) {
+                if (frame->frameShape() != QFrame::NoFrame) {
+                    m_nativeFrameShape = frame->frameShape();
+                    frame->setFrameShape(QFrame::NoFrame);
+                }
+
+            } else if (m_nativeFrameShape != QFrame::NoFrame) {
+                frame->setFrameShape(m_nativeFrameShape);
+            }
+        } else if (QLineEdit* lineEdit = qobject_cast<QLineEdit*>(m_widget)) {
+            lineEdit->setFrame(!shouldDisableNativeBorders()); 
         }
 
         // Font:
@@ -497,7 +505,7 @@ void RenderWidget::updateFromElement()
 
 void RenderWidget::paintOneBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
 {
-    bool fudge = !shouldPaintBorder();
+    bool fudge = !shouldPaintCSSBorders();
     paintBackgroundExtended(p, c, bgLayer, clipr, _tx, _ty, w, height,
                                 fudge ? 1 : borderLeft() , fudge ? 1 : borderRight(), paddingLeft(), paddingRight(),
                                 fudge ? 1 : borderTop(), fudge ? 1 : borderBottom(), paddingTop(), paddingBottom());
@@ -510,13 +518,13 @@ void RenderWidget::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
     QRect cr = r.intersected(paintInfo.r);
 
     if (qobject_cast<QAbstractScrollArea*>(m_widget) || (isRedirectedWidget() && 
-            (style()->hasBackgroundImage() || (style()->hasBackground() && shouldPaintBorder()))))
+            (style()->hasBackgroundImage() || (style()->hasBackground() && shouldPaintCSSBorders()))))
     {
         paintAllBackgrounds(paintInfo.p, style()->backgroundColor(), style()->backgroundLayers(), 
             cr, r.x(), r.y(), r.width(), r.height());
     }
 
-    if (shouldPaintBorder() && style()->hasBorder())
+    if (shouldPaintCSSBorders() && style()->hasBorder())
     {
         paintBorder(paintInfo.p, _tx, _ty, width(), height(), style());
     }
@@ -660,21 +668,25 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
  
     QPoint thePoint(tx, ty);
     QTransform t = p->worldTransform();
-    bool vte = p->viewTransformEnabled();
-    bool wme = p->worldMatrixEnabled();
-    QRect w = p->window();
-    QRect v = p->viewport();
+
     if (!buffered && t.isTranslating()) {
         thePoint.setX( thePoint.x()+ static_cast<int>(t.dx()) );
         thePoint.setY( thePoint.y()+ static_cast<int>(t.dy()) );
     }
-    QRegion rg = p->clipRegion();
-    QPaintDevice *d = p->device();
-    QPaintDevice *x = d;
-    qreal op = p->opacity();
     QPixmap* pm = 0;
+    QPaintDevice *d = p->device();
+#ifdef IN_PLACE_WIDGET_PAINTING
+    QPaintDevice *x = d;
+    bool vte = p->viewTransformEnabled();
+    bool wme = p->worldMatrixEnabled();
+    QRect w = p->window();
+    QRect v = p->viewport();
+    QRegion rg = p->clipRegion();
+    qreal op = p->opacity();
     QPen pen = p->pen();
     QBrush brush = p->brush();
+    QPainter::RenderHints rh = p->renderHints();
+#endif
     if (buffered) {
         if (!widget->size().isValid())
             return;
@@ -704,18 +716,23 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
     setInPaintEventFlag( widget );
 
     if (!buffered) {
+#ifdef IN_PLACE_WIDGET_PAINTING
         p->begin(x);
         p->setWorldTransform(t);
         p->setWindow(w);
         p->setViewport(v);
         p->setViewTransformEnabled( vte );
         p->setWorldMatrixEnabled( wme );
+        p->setRenderHints( rh );
         if (!rg.isEmpty())
             p->setClipRegion(rg);
         if (op < 1.0f)
             p->setOpacity(op);
         p->setPen(pen);
         p->setBrush(brush);
+#else
+        assert(false);
+#endif
     } else {
         // transfer results
         QPoint off(r.x(), r.y());
@@ -729,12 +746,12 @@ void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty, Q
     QPainter* const p = pI.p;
     allowWidgetPaintEvents = true;
 
-    // Qt 4.4 regression #2: 
-    //    can't use QWidget::render to directly paint widgets on the view anymore.
-    //    Results are unreliable for subrects, leaving blank squares. (cf. kde #158607)
-    //
-    bool buffered = true; // p->combinedMatrix().m22() != 1.0 || (p->device()->devType() == QInternal::Printer);
-
+    bool buffered =
+#ifdef IN_PLACE_WIDGET_PAINTING
+    p->combinedMatrix().m22() != 1.0 || (p->device()->devType() == QInternal::Printer);
+#else
+    true;
+#endif
     QRect rr = pI.r;
     rr.translate(-tx, -ty);
     const QRect r = widget->rect().intersect( rr );
