@@ -37,8 +37,8 @@
 #include <kdesktopfile.h>
 #include <klineedit.h>
 #include <klocale.h>
-#include <kiconloader.h>
 #include <kmessagebox.h>
+#include <kshell.h>
 #include <krun.h>
 #include <kstandarddirs.h>
 #include <kstringhandler.h>
@@ -366,16 +366,16 @@ KApplicationView::~KApplicationView()
 void KApplicationView::setModel(QAbstractItemModel *model)
 {
     if (d->appModel) {
-        disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
-                this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
+        disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+                this, SLOT(slotSelectionChanged(QItemSelection,QItemSelection)));
     }
 
     QTreeView::setModel(model);
 
     d->appModel = qobject_cast<KApplicationModel*>(model);
     if (d->appModel) {
-        connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
-                this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
+        connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+                this, SLOT(slotSelectionChanged(QItemSelection,QItemSelection)));
     }
 }
 
@@ -459,6 +459,7 @@ public:
 
     // slots
     void _k_slotDbClick();
+    void _k_slotFileSelected();
 
     bool saveNewApps;
     bool m_terminaldirty;
@@ -618,16 +619,17 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
   }
 
     QObject::connect(edit, SIGNAL(textChanged(QString)), q, SLOT(slotTextChanged()));
+    QObject::connect(edit, SIGNAL(urlSelected(KUrl)), q, SLOT(_k_slotFileSelected()));
 
     view = new KApplicationView(mainWidget);
     view->setModel(new KApplicationModel(view));
     topLayout->addWidget(view);
     topLayout->setStretchFactor(view, 1);
 
-    QObject::connect(view, SIGNAL(selected(QString, QString)),
-                     q, SLOT(slotSelected(QString, QString)));
-    QObject::connect(view, SIGNAL(highlighted(QString, QString)),
-                     q, SLOT(slotHighlighted(QString, QString)));
+    QObject::connect(view, SIGNAL(selected(QString,QString)),
+                     q, SLOT(slotSelected(QString,QString)));
+    QObject::connect(view, SIGNAL(highlighted(QString,QString)),
+                     q, SLOT(slotHighlighted(QString,QString)));
     QObject::connect(view, SIGNAL(doubleClicked(QModelIndex)),
                      q, SLOT(_k_slotDbClick()));
 
@@ -744,6 +746,12 @@ void KOpenWithDialogPrivate::_k_slotDbClick()
     q->accept();
 }
 
+void KOpenWithDialogPrivate::_k_slotFileSelected()
+{
+    // quote the path to avoid unescaped whitespace, backslashes, etc.
+    edit->setText(KShell::quoteArg(edit->text()));
+}
+
 void KOpenWithDialog::setSaveNewApplications(bool b)
 {
   d->saveNewApps = b;
@@ -892,6 +900,10 @@ bool KOpenWithDialogPrivate::checkAccept()
             // If we got here, we can't seem to find a service for what they wanted. Create one.
 
             QString menuId;
+#ifdef Q_WS_WIN32
+            // on windows, do not use the complete path, but only the default name.
+            serviceName = QFileInfo(serviceName).fileName();
+#endif
             QString newPath = KService::newServicePath(false /* ignored argument */, serviceName, &menuId);
             kDebug(250) << "Creating new service" << serviceName << "(" << newPath << ")" << "menuId=" << menuId;
 
