@@ -64,7 +64,6 @@ public:
     hasPendingChanges = false;
 
     window = 0;
-    m_cachedItemsJob = 0;
   }
 
     void _k_emitCachedItems(const KUrl&, bool, bool);
@@ -106,6 +105,9 @@ public:
 
     void emitChanges();
 
+    class CachedItemsJob;
+    CachedItemsJob* cachedItemsJobForUrl(const KUrl& url) const;
+
 
   KDirLister *m_parent;
 
@@ -145,8 +147,7 @@ public:
   KFileItemList *lstMimeFilteredItems, *lstRemoveItems;
 
     QWidget *window; // Main window this lister is associated with
-    class CachedItemsJob;
-    CachedItemsJob* m_cachedItemsJob;
+    QList<CachedItemsJob*> m_cachedItemsJobs;
 
     QString nameFilter; // parsed into lstFilters
 
@@ -194,7 +195,7 @@ public:
     // stop all running jobs for lister
     void stop( KDirLister *lister, bool silent = false );
     // stop just the job listing url for lister
-    void stop( KDirLister *lister, const KUrl &_url, bool silent = false );
+    void stopListingUrl( KDirLister *lister, const KUrl &_url, bool silent = false );
 
   void setAutoUpdate( KDirLister *lister, bool enable );
 
@@ -208,9 +209,12 @@ public:
     KFileItem *findByUrl(const KDirLister *lister, const KUrl &url) const;
 
     // Called by CachedItemsJob:
-    // Emits those items, for this lister and this url
-    void emitItemsFromCache(KDirLister* lister, const KFileItemList& lst, const KFileItem& rootItem,
+    // Emits the cached items, for this lister and this url
+    void emitItemsFromCache(KDirLister::Private::CachedItemsJob* job, KDirLister* lister,
                             const KUrl& _url, bool _reload, bool _emitCompleted);
+    // Called by CachedItemsJob:
+    void forgetCachedItemsJob(KDirLister::Private::CachedItemsJob* job, KDirLister* lister,
+                              const KUrl& url);
 
 public Q_SLOTS:
   /**
@@ -258,8 +262,7 @@ private:
 
     bool validUrl( const KDirLister *lister, const KUrl& _url ) const;
 
-    // helper for both stop methods
-    void stopLister(KDirLister* lister, const QString& url, KDirListerCacheDirectoryData& dirData, bool silent);
+    void stopListJob(const QString& url, bool silent);
 
     KIO::ListJob *jobForUrl( const QString& url, KIO::ListJob *not_job = 0 );
     const KUrl& joburl( KIO::ListJob *job );
@@ -463,17 +466,7 @@ struct KDirListerCacheDirectoryData
 class KDirLister::Private::CachedItemsJob : public KJob {
     Q_OBJECT
 public:
-    CachedItemsJob(KDirLister* lister, const KFileItemList& items, const KFileItem& rootItem,
-                   const KUrl& url, bool reload)
-        : KJob(lister),
-          m_lister(lister), m_url(url),
-          m_items(items), m_rootItem(rootItem),
-          m_reload(reload), m_emitCompleted(true) {
-        Q_ASSERT(lister->d->m_cachedItemsJob == 0);
-        lister->d->m_cachedItemsJob = this;
-        setAutoDelete(true);
-        start();
-    }
+    CachedItemsJob(KDirLister* lister, const KUrl& url, bool reload);
 
     /*reimp*/ void start() { QMetaObject::invokeMethod(this, "done", Qt::QueuedConnection); }
 
@@ -482,14 +475,15 @@ public:
 
     KUrl url() const { return m_url; }
 
+protected:
+    virtual bool doKill();
+
 public Q_SLOTS:
     void done();
 
 private:
     KDirLister* m_lister;
     KUrl m_url;
-    KFileItemList m_items;
-    KFileItem m_rootItem;
     bool m_reload;
     bool m_emitCompleted;
 };
