@@ -1,6 +1,7 @@
 /*
  * This file is part of the Nepomuk KDE project.
  * Copyright (C) 2006-2010 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2010-2012 Vishesh Handa <handa.vish@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,7 +22,6 @@
 #include "resourcedata.h"
 #include "resourcemanager.h"
 #include "resourcemanager_p.h"
-#include "resourcefiltermodel.h"
 #include "resource.h"
 #include "tools.h"
 #include "nie.h"
@@ -30,6 +30,8 @@
 #include "nepomukmainmodel.h"
 #include "dbusconnectionpool.h"
 #include "class.h"
+#include "dbustypes.h"
+#include "resourcewatcher.h"
 
 #include <Soprano/Statement>
 #include <Soprano/StatementIterator>
@@ -49,17 +51,19 @@
 
 #include <kdebug.h>
 #include <kurl.h>
+#include <kcomponentdata.h>
 
 using namespace Soprano;
 
-#define MAINMODEL static_cast<ResourceFilterModel*>(m_rm->m_manager->mainModel())
+#define MAINMODEL (m_rm->m_manager->mainModel())
 
 
 Nepomuk::ResourceData::ResourceData( const QUrl& uri, const QUrl& kickOffUri, const QUrl& type, ResourceManagerPrivate* rm )
     : m_uri(uri),
       m_mainType( type ),
       m_modificationMutex(QMutex::Recursive),
-      m_cacheDirty(true),
+      m_cacheDirty(false),
+      m_addedToWatcher(false),
       m_pimoThing(0),
       m_groundingOccurence(0),
       m_rm(rm)
@@ -76,10 +80,22 @@ Nepomuk::ResourceData::ResourceData( const QUrl& uri, const QUrl& kickOffUri, co
     m_rm->dataCnt.ref();
 
     if( !uri.isEmpty() ) {
+        m_cacheDirty = true;
+        m_rm->m_initializedData.insert( uri, this );
         m_kickoffUris.insert( uri );
     }
     if( !kickOffUri.isEmpty() ) {
         m_kickoffUris.insert( kickOffUri );
+
+        if( kickOffUri.scheme().isEmpty() ) {
+            // Label
+            const QString label = kickOffUri.toString();
+            m_cache.insert( Soprano::Vocabulary::NAO::identifier(), label );
+        }
+        else if( kickOffUri.scheme() != QLatin1String("nepomuk") ) {
+            // It's probably the nie:url
+            m_cache.insert( Nepomuk::Vocabulary::NIE::url(), kickOffUri );
+        }
     }
     m_rm->addToKickOffList( this, m_kickoffUris );
 }
@@ -140,7 +156,7 @@ void Nepomuk::ResourceData::setTypes( const QList<QUrl>& types )
     }
 
     // update the data store
-    MAINMODEL->updateProperty( m_uri, Soprano::Vocabulary::RDF::type(), nodes );
+    setProperty(Soprano::Vocabulary::RDF::type(), Nepomuk::Variant(types) );
 }
 
 
@@ -149,10 +165,23 @@ void Nepomuk::ResourceData::resetAll( bool isDelete )
 {
     // remove us from all caches (store() will re-insert us later if necessary)
     m_rm->mutex.lock();
-    if( !m_uri.isEmpty() )
-        m_rm->m_initializedData.remove( m_uri );
+
+    // IMPORTANT:
+    // Remove from the kickOffList before removing from the resource watcher
+    // This is required cause otherwise the Resource::fromResourceUri creates a new
+    // resource which is correctly identified to the ResourceData (this), and it is
+    // then deleted, which calls resetAll and this cycle continues.
     Q_FOREACH( const KUrl& uri, m_kickoffUris )
         m_rm->m_uriKickoffData.remove( uri );
+
+    if( !m_uri.isEmpty() ) {
+        m_rm->m_initializedData.remove( m_uri );
+        if( m_rm->m_watcher && m_addedToWatcher ) {
+            // See load() for an explanation of the QMetaObject call
+            QMetaObject::invokeMethod(m_rm->m_watcher, "removeResource", Qt::AutoConnection, Q_ARG(Nepomuk::Resource, Resource::fromResourceUri(m_uri)));
+            m_addedToWatcher = false;
+        }
+    }
     m_rm->mutex.unlock();
 
     // reset all variables
@@ -258,8 +287,37 @@ bool Nepomuk::ResourceData::store()
 
     if ( m_uri.isEmpty() ) {
         QMutexLocker rmlock(&m_rm->mutex);
-        // create a random URI and add us to the initialized data, i.e. make us "valid"
-        m_uri = m_rm->m_manager->generateUniqueUri( QString() );
+
+        if ( m_nieUrl.isValid() &&
+             m_nieUrl.isLocalFile() &&
+             m_mainType == Soprano::Vocabulary::RDFS::Resource() ) {
+            m_mainType = Nepomuk::Vocabulary::NFO::FileDataObject();
+            m_types << m_mainType;
+        }
+
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("createResource") );
+        QString app = KGlobal::mainComponent().componentName();
+        QVariantList arguments;
+
+        //FIXME: Maybe we should be setting the 'label' over here.
+        arguments << DBus::convertUriList(m_types) << QString() << QString() << app;
+        msg.setArguments( arguments );
+
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
+            //TODO: Set the error somehow
+            kWarning() << reply.errorMessage();
+            return false;
+        }
+        else if( reply.type() == QDBusMessage::ReplyMessage ) {
+            m_uri = reply.arguments().at(0).toUrl();
+        }
+
+        // Add us to the initialized data, i.e. make us "valid"
         m_rm->m_initializedData.insert( m_uri, this );
 
         // each initialized resource has to be in a kickoff list
@@ -268,53 +326,23 @@ bool Nepomuk::ResourceData::store()
             m_kickoffUris.insert( m_uri );
             m_rm->addToKickOffList( this, m_kickoffUris );
         }
-    }
-
-    QList<Statement> statements;
-
-    if ( !exists() ) {
-        // save the creation date
-        statements.append( Statement( m_uri, Soprano::Vocabulary::NAO::created(), Soprano::LiteralValue( QDateTime::currentDateTime() ) ) );
-
-        // save the kickoff identifier (other identifiers are stored via setProperty)
-        Q_FOREACH ( const KUrl & url, m_kickoffUris ) {
-            if( url.scheme().isEmpty() ) {
-                statements.append( Statement( m_uri, Soprano::Vocabulary::NAO::identifier(), LiteralValue(url.url()) ) );
-                break;
-            }
-        }
-
-        // the only situation in which determineUri keeps the kickoff URI is for file URLs.
-        if ( m_nieUrl.isValid() ) {
-            statements.append( Statement( m_uri, Nepomuk::Vocabulary::NIE::url(), m_nieUrl ) );
-            if ( m_nieUrl.isLocalFile() &&
-                 m_mainType == Soprano::Vocabulary::RDFS::Resource() ) {
-                m_mainType = Nepomuk::Vocabulary::NFO::FileDataObject();
-            }
-        }
 
         // store our grounding occurrence in case we are a thing created by the pimoThing() method
         if( m_groundingOccurence ) {
             if( m_groundingOccurence != this )
                 m_groundingOccurence->store();
-            statements.append( Statement( m_uri, Vocabulary::PIMO::groundingOccurrence(), m_groundingOccurence->uri() ) );
+            setProperty(Vocabulary::PIMO::groundingOccurrence(), Variant(m_groundingOccurence->uri()) );
+        }
+
+        foreach( const KUrl& url, m_kickoffUris ) {
+            if( url.scheme().isEmpty() )
+                setProperty( Soprano::Vocabulary::NAO::identifier(), Variant(url.url()) );
+            else
+                setProperty( Nepomuk::Vocabulary::NIE::url(), Variant(url.url()) );
         }
     }
 
-    // save type (There should be no need to save all the types since there is only one way
-    // that m_types contains more than one element: if we loaded them)
-    // The first type, however, can be set at creation time to any value
-    if ( m_mainType != Soprano::Vocabulary::RDFS::Resource() &&
-         !MAINMODEL->containsAnyStatement( m_uri, Soprano::Vocabulary::RDF::type(), m_mainType ) ) {
-        statements.append( Statement( m_uri, Soprano::Vocabulary::RDF::type(), m_mainType ) );
-    }
-
-    if ( !statements.isEmpty() ) {
-        return MAINMODEL->addStatements( statements ) == Soprano::Error::ErrorNone;
-    }
-    else {
-        return true;
-    }
+    return true;
 }
 
 
@@ -367,9 +395,34 @@ bool Nepomuk::ResourceData::load()
     if ( m_cacheDirty ) {
         m_cache.clear();
 
+        if(!m_rm->m_watcher) {
+            //
+            // The ResourceWatcher is not thread-safe. Thus, we need to ensure the safety ourselves.
+            // We do that by simply handling all RW related operations in the manager thread.
+            // This also means to invoke methods on the watcher through QMetaObject to make sure they
+            // get queued in case of calls between different threads.
+            //
+            m_rm->m_watcher = new ResourceWatcher(m_rm->m_manager);
+            m_rm->m_watcher->moveToThread(m_rm->m_manager->thread());
+            QObject::connect( m_rm->m_watcher, SIGNAL(propertyAdded(Nepomuk::Resource, Nepomuk::Types::Property, QVariant)),
+                              m_rm->m_manager, SLOT(slotPropertyAdded(Nepomuk::Resource, Nepomuk::Types::Property, QVariant)) );
+            QObject::connect( m_rm->m_watcher, SIGNAL(propertyRemoved(Nepomuk::Resource, Nepomuk::Types::Property, QVariant)),
+                              m_rm->m_manager, SLOT(slotPropertyRemoved(Nepomuk::Resource, Nepomuk::Types::Property, QVariant)) );
+            m_rm->m_watcher->addResource( Nepomuk::Resource::fromResourceUri(m_uri) );
+            QMetaObject::invokeMethod(m_rm->m_watcher, "start", Qt::AutoConnection);
+        }
+        else {
+            QMetaObject::invokeMethod(m_rm->m_watcher, "addResource", Qt::AutoConnection, Q_ARG(Nepomuk::Resource, Nepomuk::Resource::fromResourceUri(m_uri)) );
+        }
+        m_addedToWatcher = true;
+
         if ( m_uri.isValid() ) {
+            //
+            // We exclude properties that are part of the inference graph
+            // It would only pollute the user interface
+            //
             Soprano::QueryResultIterator it = MAINMODEL->executeQuery(QString("select distinct ?p ?o where { "
-                                                                              "%1 ?p ?o . "
+                                                                              "graph ?g { %1 ?p ?o . } . FILTER(?g!=<urn:crappyinference2:inferredtriples>) . "
                                                                               "}").arg(Soprano::Node::resourceToN3(m_uri)),
                                                                       Soprano::Query::QueryLanguageSparql);
             while ( it.next() ) {
@@ -381,8 +434,9 @@ bool Nepomuk::ResourceData::load()
                     }
                 }
                 else {
-                    updateKickOffLists( p, o.uri() );
-                    m_cache[p].append( Variant::fromNode( o ) );
+                    Nepomuk::Variant var = Variant::fromNode( o );
+                    updateKickOffLists( p, var );
+                    m_cache[p].append( var );
                 }
             }
 
@@ -424,39 +478,35 @@ void Nepomuk::ResourceData::setProperty( const QUrl& uri, const Nepomuk::Variant
         // step 0: make sure this resource is in the store
         QMutexLocker lock(&m_modificationMutex);
 
-        QList<Node> valueNodes;
-
-        // make sure resource values are in the store
-        if ( value.simpleType() == qMetaTypeId<Resource>() ) {
-            QList<Resource> l = value.toResourceList();
-            for( QList<Resource>::iterator resIt = l.begin(); resIt != l.end(); ++resIt ) {
-                resIt->m_data->store();
+        // update the store
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("setProperty") );
+        QVariantList varList;
+        foreach( const Nepomuk::Variant var, value.toVariantList() ) {
+            // make sure resource values are in the store
+            if( var.simpleType() == qMetaTypeId<Resource>() ) {
+                var.toResource().m_data->store();
+                varList << var.toUrl();
+            }
+            else {
+                varList << var.variant();
             }
         }
 
-        // add the actual property statements
+        msg.setArguments( QVariantList()
+                          << DBus::convertUriList(QList<QUrl>() << m_uri)
+                          << DBus::convertUri(uri)
+                          << QVariant(DBus::normalizeVariantList(varList))
+                          << KGlobal::mainComponent().componentName() );
 
-        // one-to-one Resource
-        if( value.isResource() ) {
-            valueNodes.append( value.toUrl() );
-        }
-
-        // one-to-many Resource
-        else if( value.isResourceList() ) {
-            const QList<QUrl>& l = value.toUrlList();
-            for( QList<QUrl>::const_iterator resIt = l.constBegin(); resIt != l.constEnd(); ++resIt ) {
-                valueNodes.append( *resIt );
-            }
-        }
-
-        // one-to-many literals
-        else if( value.isList() ) {
-            valueNodes = Nepomuk::valuesToRDFNodes( value );
-        }
-
-        // one-to-one literal
-        else {
-            valueNodes.append( Nepomuk::valueToRDFNode( value ) );
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
+            //TODO: Set the error somehow
+            kWarning() << reply.errorMessage();
+            return;
         }
 
         // update the cache for now
@@ -465,11 +515,57 @@ void Nepomuk::ResourceData::setProperty( const QUrl& uri, const Nepomuk::Variant
         else
             m_cache.remove(uri);
 
+        // update the kickofflists
+        updateKickOffLists( uri, value );
+    }
+}
+
+
+void Nepomuk::ResourceData::addProperty( const QUrl& uri, const Nepomuk::Variant& value )
+{
+    Q_ASSERT( uri.isValid() );
+
+    if( value.isValid() && store() ) {
+        // step 0: make sure this resource is in the store
+        QMutexLocker lock(&m_modificationMutex);
+
         // update the store
-        MAINMODEL->updateProperty( m_uri, uri, valueNodes );
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("addProperty") );
+        QVariantList varList;
+        foreach( const Nepomuk::Variant var, value.toVariantList() ) {
+            // make sure resource values are in the store
+            if( var.simpleType() == qMetaTypeId<Resource>() ) {
+                var.toResource().m_data->store();
+                varList << var.toUrl();
+            }
+            else {
+                varList << var.variant();
+            }
+        }
+
+        msg.setArguments( QVariantList()
+                          << DBus::convertUriList(QList<QUrl>() << m_uri)
+                          << DBus::convertUri(uri)
+                          << QVariant(DBus::normalizeVariantList(varList))
+                          << KGlobal::mainComponent().componentName() );
+
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
+            //TODO: Set the error somehow
+            kWarning() << reply.errorMessage();
+            return;
+        }
+
+        // update the cache for now
+        if( value.isValid() )
+            m_cache[uri].append(value);
 
         // update the kickofflists
-        updateKickOffLists( uri, value.toUrl() );
+        updateKickOffLists( uri, value );
     }
 }
 
@@ -480,23 +576,54 @@ void Nepomuk::ResourceData::removeProperty( const QUrl& uri )
     if( !m_uri.isEmpty() ) {
         QMutexLocker lock(&m_modificationMutex);
 
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("removeProperties") );
+        msg.setArguments( QVariantList()
+                          << DBus::convertUri(m_uri)
+                          << DBus::convertUri(uri)
+                          << KGlobal::mainComponent().componentName() );
+
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
+            //TODO: Set the error somehow
+            kWarning() << reply.errorMessage();
+            return;
+        }
+
+        // Update the cache
         m_cache.remove( uri );
-        MAINMODEL->removeProperty( m_uri, uri );
 
         // update the kickofflists
-        updateKickOffLists( uri, QUrl() );
+        updateKickOffLists( uri, Variant() );
     }
 }
 
 
 void Nepomuk::ResourceData::remove( bool recursive )
 {
+    Q_UNUSED(recursive)
     QMutexLocker lock(&m_modificationMutex);
 
     if( !m_uri.isEmpty() ) {
-        MAINMODEL->removeAllStatements( Statement( m_uri, Node(), Node() ) );
-        if ( recursive ) {
-            MAINMODEL->removeAllStatements( Statement( Node(), Node(), m_uri ) );
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("removeResources") );
+        // TODO: Set the flag over here
+        msg.setArguments( QVariantList()
+                          << DBus::convertUri(m_uri)
+                          << 0 /* no flags */
+                          << KGlobal::mainComponent().componentName());
+
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
+            //TODO: Set the error somehow
+            kWarning() << reply.errorMessage();
+            return;
         }
     }
 
@@ -507,7 +634,9 @@ void Nepomuk::ResourceData::remove( bool recursive )
 bool Nepomuk::ResourceData::exists()
 {
     if( m_uri.isValid() ) {
-        return MAINMODEL->containsAnyStatement( Statement( m_uri, Node(), Node() ) );
+        const QString query = QString::fromLatin1("ask { %1 ?p ?o . }")
+                              .arg( Soprano::Node::resourceToN3(m_uri) );
+        return MAINMODEL->executeQuery( query, Soprano::Query::QueryLanguageSparql ).boolValue();
     }
     else {
         return false;
@@ -589,32 +718,6 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::determineUri()
                     }
                     it.close();
                 }
-                else if( kickOffUri.scheme() == QLatin1String("file") ) {
-                    //
-                    // This is the hard part: The file may still be saved using a filex:/ URL
-                    // We have to determine if that is the case. For that we need to look if the URL
-                    // starts with any of the mounted filesystems' mount paths and then reconstruct
-                    // the filex:/ URL. Since that is quite some work which involved Solid and, thus,
-                    // DBus calls we simply call the removable storage service directly.
-                    //
-                    // If there is no resource yet we create a new uri in store()
-                    //
-
-                    //
-                    // We create a new dbus connection to protect us from multi-thread related crashes.
-                    //
-                    KUrl resourceUri = QDBusReply<QString>( QDBusInterface(QLatin1String("org.kde.nepomuk.services.nepomukremovablestorageservice"),
-                                                                           QLatin1String("/nepomukremovablestorageservice"),
-                                                                           QLatin1String("org.kde.nepomuk.RemovableStorage"),
-                                                                           DBusConnectionPool::threadConnection())
-                                                            .call( QLatin1String("resourceUriFromLocalFileUrl"),
-                                                                   kickOffUri.url() ) ).value();
-                    if( !resourceUri.isEmpty() ) {
-                        m_uri = resourceUri;
-                    }
-
-                    m_nieUrl = kickOffUri;
-                }
                 else if( kickOffUri.scheme() == QLatin1String("nepomuk") ) {
                     // for nepomuk URIs we simply use the kickoff URI as resource URI
                     m_uri = kickOffUri;
@@ -625,20 +728,19 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::determineUri()
                 }
             }
         }
-    }
 
-    //
-    // Move us to the final data hash now that the URI is known
-    //
-    if( !m_uri.isEmpty() ) {
-        QMutexLocker rmlock(&m_rm->mutex);
-
-        ResourceDataHash::iterator it = m_rm->m_initializedData.find(m_uri);
-        if( it == m_rm->m_initializedData.end() ) {
-            m_rm->m_initializedData.insert( m_uri, this );
-        }
-        else {
-            return it.value();
+        //
+        // Move us to the final data hash now that the URI is known
+        //
+        if( !m_uri.isEmpty() ) {
+            m_cacheDirty = true;
+            ResourceDataHash::iterator it = m_rm->m_initializedData.find(m_uri);
+            if( it == m_rm->m_initializedData.end() ) {
+                m_rm->m_initializedData.insert( m_uri, this );
+            }
+            else {
+                return it.value();
+            }
         }
     }
 
@@ -656,23 +758,23 @@ Nepomuk::Thing Nepomuk::ResourceData::pimoThing()
 {
     load();
     if( !m_pimoThing ) {
-        if( hasType( Vocabulary::PIMO::Thing() ) ) {
-//            kDebug() << "we are already a thing. Creating link to ourself" << m_uri << this;
-            // we are out own thing
-            m_pimoThing = new Thing(this);
-        }
-        else if( isFile() ) {
-            // files are a special case in every aspect. this includes pimo things.
-            // files are their own grounding occurrence. This makes a lot of things
-            // much simpler.
+        //
+        // We only create a new thing if we are a nie:InformationElement.
+        // All other resources will simply be converted into a pimo:Thing
+        //
+        // Files, however, are a special case in every aspect. this includes pimo things.
+        // Files are their own grounding occurrence. This makes a lot of things
+        // much simpler.
+        //
+        if( hasType( Vocabulary::PIMO::Thing() ) ||
+                isFile() ||
+                !hasType( Vocabulary::NIE::InformationElement() ) ) {
             m_pimoThing = new Thing(this);
         }
         else {
-//            kDebug() << "creating new thing for" << m_uri << m_types << this;
             m_pimoThing = new Thing();
         }
         m_pimoThing->m_data->m_groundingOccurence = this;
-//        kDebug() << "created thing" << m_pimoThing->m_data << "with grounding occurence" << m_pimoThing->m_data->m_groundingOccurence;
     }
     return *m_pimoThing;
 }
@@ -708,11 +810,13 @@ QDebug operator<<( QDebug dbg, const Nepomuk::ResourceData& data )
 }
 
 
-void Nepomuk::ResourceData::updateKickOffLists(const QUrl& prop, const QUrl& newUrl)
+void Nepomuk::ResourceData::updateKickOffLists(const QUrl& prop, const Nepomuk::Variant& v)
 {
     KUrl oldUrl;
+    KUrl newUrl;
     if( prop == Nepomuk::Vocabulary::NIE::url() ) {
         oldUrl = m_nieUrl;
+        newUrl = v.toUrl();
         m_nieUrl = newUrl;
     }
     else if( prop == Soprano::Vocabulary::NAO::identifier() ) {
@@ -722,6 +826,7 @@ void Nepomuk::ResourceData::updateKickOffLists(const QUrl& prop, const QUrl& new
                 break;
             }
         }
+        newUrl = KUrl( v.toString() );
     }
     else {
         return;
