@@ -20,6 +20,7 @@
 
 #include "popupapplet.h"
 #include "private/popupapplet_p.h"
+#include "private/dialog_p.h"
 
 #include <QApplication>
 #include <QGraphicsProxyWidget>
@@ -80,63 +81,41 @@ void PopupApplet::setPopupIcon(const QIcon &icon)
             d->icon = 0;
             setLayout(0);
             setAspectRatioMode(d->savedAspectRatio);
+            d->popupConstraintsEvent(FormFactorConstraint);
         }
 
         return;
     }
 
-    if (!d->icon) {
-        d->icon = new Plasma::IconWidget(icon, QString(), this);
-        connect(d->icon, SIGNAL(clicked()), this, SLOT(internalTogglePopup()));
-
-        QGraphicsLinearLayout *layout = new QGraphicsLinearLayout();
-        layout->setContentsMargins(0, 0, 0, 0);
-        layout->setSpacing(0);
-        layout->setOrientation(Qt::Horizontal);
-
-        setLayout(layout);
-    } else {
-        d->icon->setIcon(icon);
-    }
+    d->createIconWidget();
+    d->icon->setIcon(icon);
 }
 
 void PopupApplet::setPopupIcon(const QString &iconName)
 {
+    // Attempt 1: is it in the plasmoid package?
     if (package()) {
-        //Attempt1: is it in the plasmoid package?
         const QString file = package()->filePath("images", iconName);
         if (!file.isEmpty()) {
             setPopupIcon(KIcon(file));
             return;
         }
-    //Attempt2: is it a svg in the icons directory?
     }
+
+    // Attempt 2: is it a svg in the icons directory?
     QString name = QString("icons/") + iconName.split("-").first();
     if (!Plasma::Theme::defaultTheme()->imagePath(name).isEmpty()) {
-        if (!d->icon) {
-            d->icon = new Plasma::IconWidget(this);
-            d->icon->setSvg(name, iconName);
-            if (d->icon->svg().isEmpty()) {
-                setPopupIcon(KIcon(iconName));
-            }
-            connect(d->icon, SIGNAL(clicked()), this, SLOT(internalTogglePopup()));
-
-            QGraphicsLinearLayout *layout = new QGraphicsLinearLayout();
-            layout->setContentsMargins(0, 0, 0, 0);
-            layout->setSpacing(0);
-            layout->setOrientation(Qt::Horizontal);
-
-            setLayout(layout);
-        } else {
-            d->icon->setSvg(name, iconName);
-            if (d->icon->svg().isEmpty()) {
-                setPopupIcon(KIcon(iconName));
-            }
+        d->createIconWidget();
+        d->icon->setSvg(name, iconName);
+        if (d->icon->svg().isEmpty()) {
+            setPopupIcon(KIcon(iconName));
         }
-    // Final Attempt: use KIcon
-    } else {
-        setPopupIcon(KIcon(iconName));
+
+        return;
     }
+
+    // Final Attempt: use KIcon
+    setPopupIcon(KIcon(iconName));
 }
 
 QIcon PopupApplet::popupIcon() const
@@ -182,8 +161,8 @@ void PopupApplet::setWidget(QWidget *widget)
 
 QGraphicsWidget *PopupApplet::graphicsWidget()
 {
-    if (d->graphicsWidget != 0) {
-        return d->graphicsWidget;
+    if (d->graphicsWidget) {
+        return d->graphicsWidget.data();
     } else {
         return static_cast<Applet*>(this)->d->extender.data();
     }
@@ -194,10 +173,12 @@ void PopupApplet::setGraphicsWidget(QGraphicsWidget *graphicsWidget)
     if (d->graphicsWidget) {
         if (d->dialogPtr) {
             d->dialogPtr.data()->setGraphicsWidget(graphicsWidget);
-        } else {
+        } else if (layout())  {
             QGraphicsLinearLayout *lay = static_cast<QGraphicsLinearLayout *>(layout());
             lay->removeAt(0);
-            lay->addItem(graphicsWidget);
+            if (graphicsWidget) {
+                lay->addItem(graphicsWidget);
+            }
         }
     }
 
@@ -236,7 +217,7 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
          (f == Plasma::Vertical || f == Plasma::Horizontal))) {
         QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(q->layout());
 
-        if (icon && !icon->icon().isNull() && lay && lay->count() > 0) {
+        if (icon && lay && lay->count() > 0) {
             lay->removeAt(0);
         }
 
@@ -363,65 +344,79 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
                 q->resize(prefSize);
                 emit q->appletTransformedItself();
             }
-        //Applet on popup
         } else {
+            //Applet on popup
+            if (icon && lay) {
+                lay->addItem(icon);
+            }
+
             //kDebug() << "about to switch to a popup";
+            if (!qWidget && !gWidget) {
+                delete dialogPtr.data();
+                return;
+            }
+
+            //there was already a dialog? don't make the switch again
+            if (dialogPtr) {
+                return;
+            }
+
             if (proxy) {
                 proxy.data()->setWidget(0); // prevent it from deleting our widget!
                 delete proxy.data();
             }
 
-            if (!dialogPtr) {
-                //save the aspect ratio mode in case we drag'n drop in the Desktop later
-                savedAspectRatio = q->aspectRatioMode();
+            //save the aspect ratio mode in case we drag'n drop in the Desktop later
+            savedAspectRatio = q->aspectRatioMode();
 
-                if (icon) {
-                    icon->show();
-                    q->setAspectRatioMode(Plasma::ConstrainedSquare);
-                }
+            if (icon) {
+                icon->show();
+                q->setAspectRatioMode(Plasma::ConstrainedSquare);
+            }
 
-                Dialog *dialog = new Dialog();
-                dialogPtr = dialog;
+            Dialog *dialog = new Dialog();
+            dialog->d->appletPtr = q;
+            dialogPtr = dialog;
 
+            if (icon) {
                 dialog->setAspectRatioMode(savedAspectRatio);
+            }
 
-                //no longer use Qt::Popup since that seems to cause a lot of problem when you drag
-                //stuff out of your Dialog (extenders). Monitor WindowDeactivate events so we can
-                //emulate the same kind of behavior as Qt::Popup (close when you click somewhere
-                //else.
+            //no longer use Qt::Popup since that seems to cause a lot of problem when you drag
+            //stuff out of your Dialog (extenders). Monitor WindowDeactivate events so we can
+            //emulate the same kind of behavior as Qt::Popup (close when you click somewhere
+            //else.
 
-                if (gWidget) {
-                    Corona *corona = qobject_cast<Corona *>(gWidget->scene());
-
-                    if (corona) {
-                        corona->addOffscreenWidget(gWidget);
-                    }
-
-                    dialog->setGraphicsWidget(gWidget);
-                    gWidget->resize(gWidget->preferredSize());
-                    dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | (gWidget->windowFlags() & Qt::X11BypassWindowManagerHint));
-                } else if (qWidget) {
-                    QVBoxLayout *l_layout = new QVBoxLayout(dialog);
-                    l_layout->setSpacing(0);
-                    l_layout->setMargin(0);
-                    l_layout->addWidget(qWidget);
-                    dialog->adjustSize();
-                    dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | (qWidget->windowFlags() & Qt::X11BypassWindowManagerHint));
-                } else {
-                    dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
+            if (gWidget) {
+                Corona *corona = qobject_cast<Corona *>(gWidget->scene());
+                if (!corona) {
+                    corona = qobject_cast<Corona *>(q->scene());
                 }
 
-                restoreDialogSize();
-                KWindowSystem::setState(dialog->winId(), NET::SkipTaskbar | NET::SkipPager);
-                dialog->installEventFilter(q);
+                if (corona) {
+                    corona->addOffscreenWidget(gWidget);
+                }
 
-                QObject::connect(dialog, SIGNAL(dialogResized()), q, SLOT(dialogSizeChanged()));
-                QObject::connect(dialog, SIGNAL(dialogVisible(bool)), q, SLOT(dialogStatusChanged(bool)));
+                gWidget->show();
+                dialog->setGraphicsWidget(gWidget);
+                dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | (gWidget->windowFlags() & Qt::X11BypassWindowManagerHint));
+            } else if (qWidget) {
+                QVBoxLayout *l_layout = new QVBoxLayout(dialog);
+                l_layout->setSpacing(0);
+                l_layout->setMargin(0);
+                l_layout->addWidget(qWidget);
+                dialog->adjustSize();
+                dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | (qWidget->windowFlags() & Qt::X11BypassWindowManagerHint));
+            } else {
+                dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
             }
 
-            if (icon && lay) {
-                lay->addItem(icon);
-            }
+            restoreDialogSize();
+            KWindowSystem::setState(dialog->winId(), NET::SkipTaskbar | NET::SkipPager);
+            dialog->installEventFilter(q);
+
+            QObject::connect(dialog, SIGNAL(dialogResized()), q, SLOT(dialogSizeChanged()));
+            QObject::connect(dialog, SIGNAL(dialogVisible(bool)), q, SLOT(dialogStatusChanged(bool)));
         }
     }
 
@@ -429,13 +424,16 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
         updateDialogPosition();
     }
 
-    emit q->sizeHintChanged(Qt::PreferredSize);
+    if (icon) {
+        // emit the size hint changing stuff for our applet as we are handling
+        // the size changings
+        emit q->sizeHintChanged(Qt::PreferredSize);
+    }
 }
 
 void PopupAppletPrivate::appletActivated()
 {
-    q->setStatus(Plasma::NeedsAttentionStatus);
-    QTimer::singleShot(0, q, SLOT(showPopup()));
+    internalTogglePopup(true);
 }
 
 QSizeF PopupApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
@@ -449,7 +447,6 @@ QSizeF PopupApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) cons
         case Horizontal: {
             const int size = IconSize(KIconLoader::Panel);
             return QSizeF(size, size);
-            break;
         }
         default:
             break;
@@ -507,7 +504,15 @@ bool PopupApplet::eventFilter(QObject *watched, QEvent *event)
                 }
             }
 
-            return c->d->showAppletContextMenu(applet, static_cast<QContextMenuEvent*>(event)->globalPos());
+            KMenu desktopMenu;
+            c->d->addAppletActions(desktopMenu, applet, event);
+
+            if (!desktopMenu.isEmpty()) {
+                desktopMenu.exec(static_cast<QContextMenuEvent*>(event)->globalPos());
+                return true;
+            }
+
+            return false;
         }
     }
 
@@ -560,33 +565,57 @@ void PopupApplet::dropEvent(QGraphicsSceneDragDropEvent *event)
 
 void PopupApplet::showPopup(uint popupDuration)
 {
-    Dialog *dialog = d->dialogPtr.data();
-    if (dialog) {
-        // move the popup before its fist show, even if the show isn't triggered by
-        // a click, this should fix the first random position seen in some widgets
-        if (!dialog->isVisible()) {
-            d->internalTogglePopup();
+    // use autohideTimer to store when the next show should be
+    if (popupDuration > 0 || d->autohideTimer) {
+        if (!d->autohideTimer) {
+            d->autohideTimer = new QTimer(this);
+            d->autohideTimer->setSingleShot(true);
+            connect(d->autohideTimer, SIGNAL(timeout()), this, SLOT(hideTimedPopup()));
         }
 
-        if (popupDuration > 0) {
-            if (!d->timer) {
-                d->timer = new QTimer(this);
-                connect(d->timer, SIGNAL(timeout()), this, SLOT(hideTimedPopup()));
+        d->autohideTimer->stop();
+        d->autohideTimer->setInterval(popupDuration);
+    }
+
+    //kDebug() << "starting delayed show, duration for popup is" << popupDuration;
+    d->delayedShowTimer.start(0, this);
+}
+
+void PopupApplet::timerEvent(QTimerEvent *event)
+{
+    if (event->timerId() == d->delayedShowTimer.timerId()) {
+        d->delayedShowTimer.stop();
+        Dialog *dialog = d->dialogPtr.data();
+        if (dialog) {
+            // move the popup before its fist show, even if the show isn't triggered by
+            // a click, this should fix the first random position seen in some widgets
+            if (!dialog->isVisible()) {
+                d->internalTogglePopup();
             }
 
-            d->timer->start(popupDuration);
-        } else {
-            if (d->timer) {
-                d->timer->stop();
+            const int popupDuration = d->autohideTimer ? d->autohideTimer->interval() : 0;
+            //kDebug() << "popupDuration is:" << (d->autohideTimer ? d->autohideTimer->interval() : 0);
+            if (popupDuration > 0) {
+                d->autohideTimer->start();
+            } else if (d->autohideTimer) {
+                d->autohideTimer->stop();
             }
         }
+    } else if (event->timerId() == d->showDialogTimer.timerId()) {
+        d->showDialogTimer.stop();
+        d->showDialog();
+    } else {
+        Applet::timerEvent(event);
     }
 }
 
 void PopupApplet::hidePopup()
 {
+    d->showDialogTimer.stop();
+    d->delayedShowTimer.stop();
+
     Dialog *dialog = d->dialogPtr.data();
-    if (dialog) {
+    if (dialog && dialog->isVisible()) {
         if (location() != Floating) {
             dialog->animatedHide(locationToInverseDirection(location()));
         } else {
@@ -605,6 +634,16 @@ Plasma::PopupPlacement PopupApplet::popupPlacement() const
     return d->popupPlacement;
 }
 
+void PopupApplet::setPopupAlignment(Qt::AlignmentFlag alignment)
+{
+    d->popupAlignment = alignment;
+}
+
+Qt::AlignmentFlag PopupApplet::popupAlignment() const
+{
+    return d->popupAlignment;
+}
+
 void PopupApplet::popupEvent(bool popped)
 {
     if (Applet::d->script) {
@@ -627,14 +666,20 @@ bool PopupApplet::isPopupShowing() const
     return d->dialogPtr && d->dialogPtr.data()->isVisible();
 }
 
+bool PopupApplet::isIconified() const
+{
+    return d->dialogPtr;
+}
+
 PopupAppletPrivate::PopupAppletPrivate(PopupApplet *applet)
         : q(applet),
           icon(0),
           widget(0),
-          graphicsWidget(0),
           popupPlacement(Plasma::FloatingPopup),
+          popupAlignment(Qt::AlignLeft),
           savedAspectRatio(Plasma::InvalidAspectRatioMode),
-          timer(0),
+          autohideTimer(0),
+          preShowStatus(UnknownStatus),
           popupLostFocus(false),
           passive(false)
 {
@@ -663,15 +708,22 @@ void PopupAppletPrivate::iconSizeChanged(int group)
     }
 }
 
-void PopupAppletPrivate::internalTogglePopup()
+void PopupAppletPrivate::internalTogglePopup(bool fromActivatedSignal)
 {
-    if (timer) {
-        timer->stop();
+    if (autohideTimer) {
+        autohideTimer->stop();
     }
 
+    delayedShowTimer.stop();
+
     Plasma::Dialog *dialog = dialogPtr.data();
     if (!dialog) {
         q->setFocus(Qt::ShortcutFocusReason);
+        if (!fromActivatedSignal) {
+            QObject::disconnect(q, SIGNAL(activate()), q, SLOT(appletActivated()));
+            emit q->activate();
+            QObject::connect(q, SIGNAL(activate()), q, SLOT(appletActivated()));
+        }
         return;
     }
 
@@ -679,10 +731,6 @@ void PopupAppletPrivate::internalTogglePopup()
         return;
     }
 
-    if (timer) {
-        timer->stop();
-    }
-
     if (dialog->isVisible()) {
         if (q->location() != Floating) {
             dialog->animatedHide(locationToInverseDirection(q->location()));
@@ -696,32 +744,49 @@ void PopupAppletPrivate::internalTogglePopup()
             q->graphicsWidget() == static_cast<Applet*>(q)->d->extender.data() &&
             static_cast<Applet*>(q)->d->extender.data()->isEmpty()) {
             // we have nothing to show, so let's not.
+            if (!fromActivatedSignal) {
+                QObject::disconnect(q, SIGNAL(activate()), q, SLOT(appletActivated()));
+                emit q->activate();
+                QObject::connect(q, SIGNAL(activate()), q, SLOT(appletActivated()));
+            }
             return;
         }
 
         ToolTipManager::self()->hide(q);
-        updateDialogPosition();
+        showDialogTimer.start(0, q);
+    }
+}
 
-        KWindowSystem::setOnAllDesktops(dialog->winId(), true);
-        KWindowSystem::setState(dialog->winId(), NET::SkipTaskbar | NET::SkipPager);
+void PopupAppletPrivate::showDialog()
+{
+    Plasma::Dialog *dialog = dialogPtr.data();
+    if (!dialog) {
+        return;
+    }
 
+    updateDialogPosition();
+
+    KWindowSystem::setOnAllDesktops(dialog->winId(), true);
+    KWindowSystem::setState(dialog->winId(), NET::SkipTaskbar | NET::SkipPager);
+
+    if (icon) {
         dialog->setAspectRatioMode(savedAspectRatio);
+    }
 
-        if (q->location() != Floating) {
-            dialog->animatedShow(locationToDirection(q->location()));
-        } else {
-            dialog->show();
-        }
+    if (q->location() != Floating) {
+        dialog->animatedShow(locationToDirection(q->location()));
+    } else {
+        dialog->show();
+    }
 
-        if (!(dialog->windowFlags() & Qt::X11BypassWindowManagerHint)) {
-            KWindowSystem::activateWindow(dialog->winId());
-        }
+    if (!(dialog->windowFlags() & Qt::X11BypassWindowManagerHint)) {
+        KWindowSystem::activateWindow(dialog->winId());
     }
 }
 
 void PopupAppletPrivate::hideTimedPopup()
 {
-    timer->stop();
+    autohideTimer->stop();
     q->hidePopup();
 }
 
@@ -730,6 +795,7 @@ void PopupAppletPrivate::clearPopupLostFocus()
     if (!icon || !icon->isDown()) {
         q->hidePopup();
     }
+
     popupLostFocus = false;
 }
 
@@ -748,16 +814,51 @@ void PopupAppletPrivate::dialogSizeChanged()
         sizeGroup.writeEntry("DialogHeight", dialog->height());
         sizeGroup.writeEntry("DialogWidth", dialog->width());
 
-        updateDialogPosition();
+        updateDialogPosition(!dialog->isUserResizing());
 
         emit q->configNeedsSaving();
         emit q->appletTransformedByUser();
     }
 }
 
-void PopupAppletPrivate::dialogStatusChanged(bool status)
+void PopupAppletPrivate::dialogStatusChanged(bool shown)
 {
-    q->popupEvent(status);
+    if (shown) {
+        preShowStatus = q->status();
+        q->setStatus(NeedsAttentionStatus);
+        QObject::connect(q, SIGNAL(newStatus(Plasma::ItemStatus)),
+                         q, SLOT(statusChangeWhileShown(Plasma::ItemStatus)),
+                         Qt::UniqueConnection);
+    } else {
+        QObject::disconnect(q, SIGNAL(newStatus(Plasma::ItemStatus)),
+                            q, SLOT(statusChangeWhileShown(Plasma::ItemStatus)));
+        q->setStatus(preShowStatus);
+    }
+
+    q->popupEvent(shown);
+}
+
+void PopupAppletPrivate::statusChangeWhileShown(Plasma::ItemStatus status)
+{
+    preShowStatus = status;
+}
+
+void PopupAppletPrivate::createIconWidget()
+{
+    if (icon) {
+        return;
+    }
+
+    icon = new Plasma::IconWidget(q);
+    QObject::connect(icon, SIGNAL(clicked()), q, SLOT(internalTogglePopup()));
+
+    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout();
+    layout->setContentsMargins(0, 0, 0, 0);
+    layout->setSpacing(0);
+    layout->setOrientation(Qt::Horizontal);
+    layout->addItem(icon);
+    layout->setAlignment(icon, Qt::AlignCenter);
+    q->setLayout(layout);
 }
 
 void PopupAppletPrivate::restoreDialogSize()
@@ -776,9 +877,10 @@ void PopupAppletPrivate::restoreDialogSize()
 
     int preferredWidth = 0;
     int preferredHeight = 0;
-    if (dialog->graphicsWidget()) {
-        preferredWidth = dialog->graphicsWidget()->preferredSize().width();
-        preferredHeight = dialog->graphicsWidget()->preferredSize().height();
+    QGraphicsWidget *gWidget = dialog->graphicsWidget();
+    if (gWidget) {
+        preferredWidth = gWidget->preferredSize().width();
+        preferredHeight = gWidget->preferredSize().height();
     }
 
     const int width = qMin(sizeGroup.readEntry("DialogWidth", preferredWidth),
@@ -796,10 +898,13 @@ void PopupAppletPrivate::restoreDialogSize()
 
     if (saved.width() != dialog->width() || saved.height() != dialog->height()) {
         dialog->resize(saved);
+       /*if (gWidget) {
+         gWidget->resize(saved);
+       }*/
     }
 }
 
-void PopupAppletPrivate::updateDialogPosition()
+void PopupAppletPrivate::updateDialogPosition(bool move)
 {
     Plasma::Dialog *dialog = dialogPtr.data();
     if (!dialog) {
@@ -816,85 +921,56 @@ void PopupAppletPrivate::updateDialogPosition()
         return;
     }
 
-    QSize s = dialog->size();
-    QPoint pos = view->mapFromScene(q->scenePos());
+    const QPoint appletPos = view->mapToGlobal(view->mapFromScene(q->scenePos()));
 
-    if (!q->containment() || view == q->containment()->view()) {
-        pos = corona->popupPosition(q, s);
-    } else {
-        pos = corona->popupPosition(q->parentItem(), s);
+    QPoint dialogPos = dialog->pos();
+    if (move) {
+        if (!q->containment() || view == q->containment()->view()) {
+            dialogPos = corona->popupPosition(q, dialog->size(), popupAlignment);
+        } else {
+            dialogPos = corona->popupPosition(q->parentItem(), dialog->size(), popupAlignment);
+        }
     }
 
     bool reverse = false;
     if (q->formFactor() == Plasma::Vertical) {
-        if (view->mapToGlobal(view->mapFromScene(q->scenePos())).y() + q->size().height()/2 < pos.y() + dialog->size().width()/2) {
-            reverse = true;
-        }
+        reverse = (appletPos.y() + (q->size().height() / 2)) < (dialogPos.y() + (dialog->size().height() / 2));
+        dialog->setMinimumResizeLimits(-1, appletPos.y(), -1, appletPos.y() + q->size().height());
     } else {
-        if (view->mapToGlobal(view->mapFromScene(q->scenePos())).x() + q->size().width()/2 < pos.x() + dialog->size().width()/2) {
-            reverse = true;
-        }
+        reverse = (appletPos.x() + (q->size().width() / 2)) < (dialogPos.x() + (dialog->size().width() / 2));
+        dialog->setMinimumResizeLimits(appletPos.x(), -1, appletPos.x() + q->size().width(), -1);
     }
 
+    Dialog::ResizeCorners resizeCorners = Dialog::NoCorner;
     switch (q->location()) {
     case BottomEdge:
-        if (pos.x() >= q->pos().x()) {
-            dialog->setResizeHandleCorners(Dialog::NorthEast);
-        } else {
-            dialog->setResizeHandleCorners(Dialog::NorthWest);
-        }
-
-        if (reverse) {
-            popupPlacement = Plasma::TopPosedLeftAlignedPopup;
-        } else {
-            popupPlacement = Plasma::TopPosedRightAlignedPopup;
-        }
+        resizeCorners = Dialog::NorthEast | Dialog::NorthWest;
+        popupPlacement = reverse ? TopPosedLeftAlignedPopup : TopPosedRightAlignedPopup;
         break;
     case TopEdge:
-        if (pos.x() >= q->pos().x()) {
-            dialog->setResizeHandleCorners(Dialog::SouthEast);
-        } else {
-            dialog->setResizeHandleCorners(Dialog::SouthWest);
-        }
-
-        if (reverse) {
-            popupPlacement = Plasma::BottomPosedLeftAlignedPopup;
-        } else {
-            popupPlacement = Plasma::BottomPosedRightAlignedPopup;
-        }
+        resizeCorners = Dialog::SouthEast | Dialog::SouthWest;
+        popupPlacement = reverse ? Plasma::BottomPosedLeftAlignedPopup : Plasma::BottomPosedRightAlignedPopup;
         break;
     case LeftEdge:
-        if (pos.y() >= q->pos().y()) {
-            dialog->setResizeHandleCorners(Dialog::SouthEast);
-        } else {
-            dialog->setResizeHandleCorners(Dialog::NorthEast);
-        }
-
-        if (reverse) {
-            popupPlacement = Plasma::RightPosedTopAlignedPopup;
-        } else {
-            popupPlacement = Plasma::RightPosedBottomAlignedPopup;
-        }
+        resizeCorners = Dialog::SouthEast | Dialog::NorthEast;
+        popupPlacement = reverse ? RightPosedTopAlignedPopup : RightPosedBottomAlignedPopup;
         break;
 
     case RightEdge:
-        if (pos.y() >= q->pos().y()) {
-            dialog->setResizeHandleCorners(Dialog::SouthWest);
-        } else {
-            dialog->setResizeHandleCorners(Dialog::NorthWest);
-        }
-
-        if (reverse) {
-            popupPlacement = Plasma::LeftPosedTopAlignedPopup;
-        } else {
-            popupPlacement = Plasma::LeftPosedBottomAlignedPopup;
-        }
+        resizeCorners = Dialog::SouthWest | Dialog::NorthWest;
+        popupPlacement = reverse ? LeftPosedTopAlignedPopup : LeftPosedBottomAlignedPopup;
         break;
+
     default:
-        dialog->setResizeHandleCorners(Dialog::NorthEast);
+        popupPlacement = FloatingPopup;
+        resizeCorners = Dialog::All;
+        break;
     }
 
-    dialog->move(pos);
+    dialog->setResizeHandleCorners(resizeCorners);
+    if (move) {
+        dialog->move(dialogPos);
+    }
 }
 
 } // Plasma namespace
