@@ -1,20 +1,21 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2008 Jeff Mitchell <mitchell@kde.org>
+/*
+    Copyright 2008 Jeff Mitchell <mitchell@kde.org>
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    Lesser General Public License for more details.
 
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef SOLID_BACKENDS_WMI_WMIQUERY_H
@@ -23,6 +24,8 @@
 #include <QtCore/QDebug>
 #include <QtCore/QVariant>
 #include <QtCore/QList>
+#include <QtCore/QAtomicInt>
+#include <QtCore/QSharedPointer>
 
 #include <solid/solid_export.h>
 
@@ -51,22 +54,28 @@ class WmiQuery
 public:
     class Item {
     public:
-        Item(IWbemClassObject *p) : m_p(p) {}
-        ~Item() {} // how to delete the pointer ?
-        QString getProperty(const QString &property );
+        Item(IWbemClassObject *p);
+        Item(const Item& other);
+        Item& operator=(const Item& other);
+        ~Item();
 
-    private: 
-        IWbemClassObject * m_p;     
+        QString getProperty(const QString &property) const;
+
+    private:
+        Item() {}
+        // QSharedPointer alone doesn't help because we need to call Release()
+        IWbemClassObject* m_p;
+        QSharedPointer<QAtomicInt> m_int;
     };
 
-    typedef QList<Item*> ItemList;
+    typedef QList<Item> ItemList;
 
     WmiQuery();
     ~WmiQuery();
     ItemList sendQuery( const QString &wql );
     bool isLegit() const;
 	static WmiQuery &instance();
-    
+
 private:
     bool m_failed;
     bool m_bNeedUninit;
